@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Angle.h"
@@ -27,10 +30,28 @@ namespace {
 	// turning 6 degrees per time step. If the Angle lookup is 2^16 steps, then 6
 	// degrees is 1092 steps, and your turn speed is accurate to +- 0.05%. That seems
 	// plenty accurate to me. At that step size, the lookup table is exactly 1 MB.
-	const int32_t STEPS = 0x10000;
-	const int32_t MASK = STEPS - 1;
-	const double DEG_TO_STEP = STEPS / 360.;
-	const double STEP_TO_RAD = PI / (STEPS / 2);
+	constexpr int32_t STEPS = 0x10000;
+	constexpr int32_t MASK = STEPS - 1;
+	constexpr double DEG_TO_STEP = STEPS / 360.;
+	constexpr double STEP_TO_RAD = PI / (STEPS / 2);
+
+	vector<Point> InitUnitCache()
+	{
+		vector<Point> cache;
+		cache.reserve(STEPS);
+		for(int i = 0; i < STEPS; ++i)
+		{
+			const double radians = i * STEP_TO_RAD;
+			// The graphics use the usual screen coordinate system, meaning that
+			// positive Y is down rather than up. Angles are clock angles, i.e.
+			// 0 is 12:00 and angles increase in the clockwise direction. So, an
+			// angle of 0 degrees is pointing in the direction (0, -1).
+			cache.emplace_back(sin(radians), -cos(radians));
+		}
+		return cache;
+	}
+
+	const vector<Point> unitCache = InitUnitCache();
 }
 
 
@@ -44,26 +65,18 @@ Angle Angle::Random()
 
 
 // Get a random angle between 0 and the given number of degrees.
-Angle Angle::Random(double range)
+Angle Angle::Random(const double range)
 {
 	// The given range would have to be about 22.6 million degrees to overflow
 	// the size of a 32-bit int, which should never happen in normal usage.
-	uint32_t mod = static_cast<uint32_t>(fabs(range) * DEG_TO_STEP) + 1;
-	return Angle(mod ? static_cast<int32_t>(Random::Int(mod)) : 0);
-}
-
-
-
-// Default constructor: generates an angle pointing straight up.
-Angle::Angle()
-	: angle(0)
-{
+	const uint32_t mod = static_cast<uint32_t>(fabs(range) * DEG_TO_STEP) + 1;
+	return Angle(mod ? static_cast<int32_t>(Random::Int(mod)) & MASK : 0);
 }
 
 
 
 // Construct an Angle from the given angle in degrees.
-Angle::Angle(double degrees)
+Angle::Angle(const double degrees) noexcept
 	: angle(llround(degrees * DEG_TO_STEP) & MASK)
 {
 	// Make sure llround does not overflow with the values of System::SetDate.
@@ -75,7 +88,7 @@ Angle::Angle(double degrees)
 
 
 // Construct an angle pointing in the direction of the given vector.
-Angle::Angle(const Point &point)
+Angle::Angle(const Point &point) noexcept
 	: Angle(TO_DEG * atan2(point.X(), -point.Y()))
 {
 }
@@ -125,25 +138,24 @@ Angle Angle::operator-() const
 
 
 
+bool Angle::operator==(const Angle &other) const
+{
+	return angle == other.angle;
+}
+
+
+
+bool Angle::operator!=(const Angle &other) const
+{
+	return angle != other.angle;
+}
+
+
+
 // Get a unit vector in the direction of this angle.
 Point Angle::Unit() const
 {
-	// The very first time this is called, create a lookup table of unit vectors.
-	static vector<Point> cache;
-	if(cache.empty())
-	{
-		cache.reserve(STEPS);
-		for(int i = 0; i < STEPS; ++i)
-		{
-			double radians = i * STEP_TO_RAD;
-			// The graphics use the usual screen coordinate system, meaning that
-			// positive Y is down rather than up. Angles are clock angles, i.e.
-			// 0 is 12:00 and angles increase in the clockwise direction. So, an
-			// angle of 0 degrees is pointing in the direction (0, -1).
-			cache.emplace_back(sin(radians), -cos(radians));
-		}
-	}
-	return cache[angle];
+	return unitCache[angle];
 }
 
 
@@ -158,13 +170,13 @@ double Angle::Degrees() const
 }
 
 
-	
+
 // Return a point rotated by this angle around (0, 0).
 Point Angle::Rotate(const Point &point) const
 {
 	// If using the normal mathematical coordinate system, this would be easier.
 	// Since we're not, the math is a tiny bit less elegant:
-	Point unit = Unit();
+	const Point unit = Unit();
 	return Point(-unit.Y() * point.X() - unit.X() * point.Y(),
 		-unit.Y() * point.Y() + unit.X() * point.X());
 }
@@ -172,7 +184,7 @@ Point Angle::Rotate(const Point &point) const
 
 
 // Constructor using Angle's internal representation.
-Angle::Angle(int32_t angle)
+Angle::Angle(const int32_t angle)
 	: angle(angle)
 {
 }
