@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef ASTEROID_FIELD_H_
@@ -15,8 +18,9 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "Angle.h"
 #include "Body.h"
-#include "Minable.h"
+#include "CollisionSet.h"
 #include "Point.h"
+#include "WeightedList.h"
 
 #include <list>
 #include <memory>
@@ -24,8 +28,11 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <vector>
 
 class DrawList;
+class Flotsam;
+class Minable;
 class Projectile;
 class Sprite;
+class Visual;
 
 
 
@@ -39,46 +46,48 @@ class Sprite;
 // are hit by a projectile.
 class AsteroidField {
 public:
+	// Constructor, to set up the collision set parameters.
+	AsteroidField();
+
 	// Reset the asteroid field (typically because you entered a new system).
 	void Clear();
 	void Add(const std::string &name, int count, double energy = 1.);
-	void Add(const Minable *minable, int count, double energy = 1., double beltRadius = 1500.);
-	
-	// Move all the asteroids forward one time step.
-	void Step(std::list<Effect> &effects, std::list<std::shared_ptr<Flotsam>> &flotsam);
+	void Add(const Minable *minable, int count, double energy, const WeightedList<double> &belts);
+
+	// Move all the asteroids forward one time step, and populate the asteroid and minable collision sets.
+	void Step(std::vector<Visual> &visuals, std::list<std::shared_ptr<Flotsam>> &flotsam, int step);
 	// Draw the asteroid field, with the field of view centered on the given point.
 	void Draw(DrawList &draw, const Point &center, double zoom) const;
-	// Check if the given projectile has hit any of the asteroids. The current
-	// time step must be given, so we know what animation frame each asteroid is
-	// on. If there is a collision the asteroid's velocity is returned so the
-	// projectile's hit effects can take it into account. The return value is
-	// how far along the projectile's path it should be clipped.
-	double Collide(const Projectile &projectile, int step, double closestHit, Point *hitVelocity = nullptr);
-	
+	// Check if the given projectile has hit any of the asteroids, using the information
+	// in the collision sets. If a collision occurs, returns a pointer to the hit body.
+	Body *Collide(const Projectile &projectile, double *closestHit);
+
 	// Get the list of minable asteroids.
 	const std::list<std::shared_ptr<Minable>> &Minables() const;
-	
-	
+
+
 private:
 	// This class represents an asteroid that cannot be destroyed or even
 	// deflected from its trajectory, and that repeats every 4096 pixels.
 	class Asteroid : public Body {
 	public:
 		Asteroid(const Sprite *sprite, double energy);
-		
+
 		void Step();
 		void Draw(DrawList &draw, const Point &center, double zoom) const;
-		double Collide(const Projectile &projectile, int step) const;
-		
+
 	private:
 		Angle spin;
 		Point size;
 	};
-	
-	
+
+
 private:
 	std::vector<Asteroid> asteroids;
 	std::list<std::shared_ptr<Minable>> minables;
+
+	CollisionSet asteroidCollisions;
+	CollisionSet minableCollisions;
 };
 
 
