@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef BOARDING_PANEL_H_
@@ -34,18 +37,18 @@ class Ship;
 class BoardingPanel : public Panel {
 public:
 	BoardingPanel(PlayerInfo &player, const std::shared_ptr<Ship> &victim);
-	
+
 	virtual void Draw() override;
-	
-	
+
+
 protected:
 	// Overrides from Panel.
-	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command) override;
+	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress) override;
 	virtual bool Click(int x, int y, int clicks) override;
 	virtual bool Drag(double dx, double dy) override;
 	virtual bool Scroll(double dx, double dy) override;
-	
-	
+
+
 private:
 	// You can't exit this dialog if you are in the middle of combat.
 	bool CanExit() const;
@@ -55,8 +58,11 @@ private:
 	bool CanCapture() const;
 	// Check if you are in the midst of hand to hand combat.
 	bool CanAttack() const;
-	
-	
+
+	// Handle the keyboard scrolling and selection in the panel list.
+	void DoKeyboardNavigation(const SDL_Keycode key);
+
+
 private:
 	// This class represents one item in the list of outfits you can plunder.
 	class Plunder {
@@ -64,16 +70,16 @@ private:
 		// Plunder can be either outfits or commodities.
 		Plunder(const std::string &commodity, int count, int unitValue);
 		Plunder(const Outfit *outfit, int count);
-		
+
 		// Sort by value per ton of mass.
 		bool operator<(const Plunder &other) const;
-		
+
 		// Check how many of this item are left un-plundered. Once this is zero,
 		// the item can be removed from the list.
 		int Count() const;
 		// Get the value of each unit of this plunder item.
 		int64_t UnitValue() const;
-		
+
 		// Get the name of this item. If it is a commodity, this is its name.
 		const std::string &Name() const;
 		// Get the mass, in the format "<count> x <unit mass>". If this is a
@@ -82,7 +88,7 @@ private:
 		const std::string &Size() const;
 		// Get the total value (unit value times count) as a string.
 		const std::string &Value() const;
-		
+
 		// If this is an outfit, get the outfit. Otherwise, this returns null.
 		const Outfit *GetOutfit() const;
 		// Find out how many of these I can take if I have this amount of cargo
@@ -90,11 +96,11 @@ private:
 		bool CanTake(const Ship &ship) const;
 		// Take some or all of this plunder item.
 		void Take(int count);
-		
+
 	private:
 		void UpdateStrings();
 		double UnitMass() const;
-		
+
 	private:
 		std::string name;
 		const Outfit *outfit;
@@ -103,17 +109,17 @@ private:
 		std::string size;
 		std::string value;
 	};
-	
+
 private:
 	PlayerInfo &player;
 	std::shared_ptr<Ship> you;
 	std::shared_ptr<Ship> victim;
-	
+
 	// List of items you can plunder.
 	std::vector<Plunder> plunder;
 	int selected = 0;
 	double scroll = 0.;
-	
+
 	bool playerDied = false;
 	bool isCapturing = false;
 	bool isFirstCaptureAction = true;
@@ -123,6 +129,9 @@ private:
 	CaptureOdds defenseOdds;
 	// These messages are shown to report the results of hand to hand combat.
 	std::vector<std::string> messages;
+
+	// Whether or not the ship can be captured.
+	bool canCapture = false;
 };
 
 
