@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef CARGO_HOLD_H_
@@ -19,6 +22,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 class DataNode;
 class DataWriter;
+class Government;
 class Mission;
 class Outfit;
 class System;
@@ -34,30 +38,33 @@ class System;
 class CargoHold {
 public:
 	void Clear();
-	
+
 	// Load the cargo manifest from a DataFile. This must be done after the
 	// GameData is loaded, so that the sizes of any outfits are known.
 	void Load(const DataNode &node);
 	// Save the cargo manifest to a file.
 	void Save(DataWriter &out) const;
-	
+
 	// Set the capacity of this cargo hold.
 	void SetSize(int tons);
 	int Size() const;
 	int Free() const;
+	double FreePrecise() const;
 	int Used() const;
+	double UsedPrecise() const;
 	int CommoditiesSize() const;
 	int OutfitsSize() const;
+	double OutfitsSizePrecise() const;
 	bool HasOutfits() const;
 	int MissionCargoSize() const;
 	bool HasMissionCargo() const;
 	bool IsEmpty() const;
-	
+
 	// Set the number of free bunks for passengers.
 	void SetBunks(int count);
 	int BunksFree() const;
 	int Passengers() const;
-	
+
 	// Normal cargo:
 	int Get(const std::string &commodity) const;
 	// Spare outfits:
@@ -65,14 +72,14 @@ public:
 	// Mission cargo:
 	int Get(const Mission *mission) const;
 	int GetPassengers(const Mission *mission) const;
-	
+
 	const std::map<std::string, int> &Commodities() const;
 	const std::map<const Outfit *, int> &Outfits() const;
 	// Note: some missions may have cargo that takes up 0 space, but should
 	// still show up on the cargo listing.
 	const std::map<const Mission *, int> &MissionCargo() const;
 	const std::map<const Mission *, int> &PassengerList() const;
-	
+
 	// For all the transfer functions, the "other" can be null if you simply want
 	// the commodity to "disappear" or, if the "amount" is negative, to have an
 	// unlimited supply. The return value is the actual number transferred.
@@ -83,33 +90,36 @@ public:
 	// Transfer as much as the given cargo hold has capacity for. The priority is
 	// first mission cargo, then spare outfits, then ordinary commodities.
 	void TransferAll(CargoHold &to, bool transferPassengers = true);
-	
+
 	// These functions do the same thing as Transfer() with no destination
 	// specified, but they have clearer names to make the code more readable.
 	int Add(const std::string &commodity, int amount = 1);
 	int Add(const Outfit *outfit, int amount = 1);
 	int Remove(const std::string &commodity, int amount = 1);
 	int Remove(const Outfit *outfit, int amount = 1);
-	
+
 	// Add or remove any cargo or passengers associated with the given mission.
 	void AddMissionCargo(const Mission *mission);
 	void RemoveMissionCargo(const Mission *mission);
-	
+
 	// Get the total value of all this cargo, in the given system.
 	int64_t Value(const System *system) const;
-	
+
 	// If anything you are carrying is illegal, return the maximum fine you can
 	// be charged for any illegal outfits plus the sum of the fines for all
 	// missions. If the returned value is negative, you are carrying something so
 	// bad that it warrants a death sentence.
-	int IllegalCargoFine() const;
-	
-	
+	int IllegalCargoFine(const Government *government) const;
+
+	// Returns the amount tons of illegal cargo.
+	int IllegalCargoAmount() const;
+
+
 private:
 	// Use -1 to indicate unlimited capacity.
 	int size = -1;
 	int bunks = -1;
-	
+
 	// Track how many objects of each type are being carried:
 	std::map<std::string, int> commodities;
 	std::map<const Outfit *, int> outfits;
