@@ -29,6 +29,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 class Color;
 class Conversation;
+class Mission;
 class PlayerInfo;
 class Point;
 class Ship;
@@ -43,8 +44,8 @@ class System;
 class ConversationPanel : public Panel {
 public:
 	ConversationPanel(PlayerInfo &player, const Conversation &conversation,
-		const System *system = nullptr, const std::shared_ptr<Ship> &ship = nullptr,
-		bool useTransactions = false);
+		const Mission *caller = nullptr, const System *system = nullptr,
+		const std::shared_ptr<Ship> &ship = nullptr, bool useTransactions = false);
 
 template <class T>
 	void SetCallback(T *t, void (T::*fun)(int));
@@ -89,7 +90,7 @@ private:
 		// Get the height of this paragraph.
 		int Height() const;
 		// Get the "center point" of this paragraph. This is for drawing a
-		// highlight under paragraphcs that represent choices.
+		// highlight under paragraphs that represent choices.
 		Point Center() const;
 		// Draw this paragraph at the given point, and return the point that the
 		// next paragraph below this one should be drawn at.
@@ -108,6 +109,9 @@ private:
 	// Reference to the player, to apply any changes to them.
 	PlayerInfo &player;
 
+	// A pointer to the mission that called this conversation.
+	const Mission *caller = nullptr;
+
 	// Should we use a PlayerInfo transaction to prevent save-load glitches?
 	bool useTransactions = false;
 
