@@ -1,5 +1,5 @@
 /* Dialog.cpp
-Copyright (c) 2014 by Michael Zahniser
+Copyright (c) 2014-2020 by Michael Zahniser
 
 Endless Sky is free software: you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Dialog.h"
@@ -15,9 +18,11 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Color.h"
 #include "Command.h"
 #include "Conversation.h"
+#include "DataNode.h"
+#include "text/DisplayText.h"
 #include "FillShader.h"
-#include "Font.h"
-#include "FontSet.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
 #include "GameData.h"
 #include "MapDetailPanel.h"
 #include "PlayerInfo.h"
@@ -29,16 +34,15 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "UI.h"
 
 #include <cmath>
-#include <functional>
 
 using namespace std;
 
 namespace {
-	static const int WIDTH = 250;
-	
+	const int WIDTH = 250;
+
 	// Map any conceivable numeric keypad keys to their ASCII values. Most of
 	// these will presumably only exist on special programming keyboards.
-	static const map<SDL_Keycode, char> KEY_MAP = {
+	const map<SDL_Keycode, char> KEY_MAP = {
 		{SDLK_KP_0, '0'},
 		{SDLK_KP_1, '1'},
 		{SDLK_KP_2, '2'},
@@ -81,22 +85,32 @@ namespace {
 }
 
 
+Dialog::Dialog(function<void()> okFunction, const string &message, Truncate truncate, bool canCancel, bool okIsActive)
+	: voidFun(okFunction)
+{
+	Init(message, truncate, canCancel, false);
+	this->okIsActive = okIsActive;
+}
+
+
 
 // Dialog that has no callback (information only). In this form, there is
 // only an "ok" button, not a "cancel" button.
-Dialog::Dialog(const string &text)
+Dialog::Dialog(const string &text, Truncate truncate, bool allowsFastForward)
+	: allowsFastForward(allowsFastForward)
 {
-	Init(text, false);
+	Init(text, truncate, false);
 }
 
 
 
 // Mission accept / decline dialog.
-Dialog::Dialog(const string &text, PlayerInfo &player, const System *system)
+Dialog::Dialog(const string &text, PlayerInfo &player, const System *system, Truncate truncate, bool allowsFastForward)
 	: intFun(bind(&PlayerInfo::MissionCallback, &player, placeholders::_1)),
+	allowsFastForward(allowsFastForward),
 	system(system), player(&player)
 {
-	Init(text, true, true);
+	Init(text, truncate, true, true);
 }
 
 
@@ -105,22 +119,22 @@ Dialog::Dialog(const string &text, PlayerInfo &player, const System *system)
 void Dialog::Draw()
 {
 	DrawBackdrop();
-	
+
 	const Sprite *top = SpriteSet::Get("ui/dialog top");
 	const Sprite *middle = SpriteSet::Get("ui/dialog middle");
 	const Sprite *bottom = SpriteSet::Get("ui/dialog bottom");
 	const Sprite *cancel = SpriteSet::Get("ui/dialog cancel");
-	
+
 	// Get the position of the top of this dialog, and of the text and input.
-	Point pos(0., (top->Height() + height * middle->Height() + bottom->Height()) * -.5);
+	Point pos(0., (top->Height() + height * middle->Height() + bottom->Height()) * -.5f);
 	Point textPos(WIDTH * -.5 + 10., pos.Y() + 20.);
 	Point inputPos = Point(0., -70.) - pos;
-	
+
 	// Draw the top section of the dialog box.
 	pos.Y() += top->Height() * .5;
 	SpriteShader::Draw(top, pos);
 	pos.Y() += top->Height() * .5;
-	
+
 	// The middle section is duplicated depending on how long the text is.
 	for(int i = 0; i < height; ++i)
 	{
@@ -128,17 +142,18 @@ void Dialog::Draw()
 		SpriteShader::Draw(middle, pos);
 		pos.Y() += middle->Height() * .5;
 	}
-	
+
 	// Draw the bottom section.
 	const Font &font = FontSet::Get(14);
 	pos.Y() += bottom->Height() * .5;
 	SpriteShader::Draw(bottom, pos);
 	pos.Y() += bottom->Height() * .5 - 25.;
-	
+
 	// Draw the buttons, including optionally the cancel button.
-	Color bright = *GameData::Colors().Get("bright");
-	Color dim = *GameData::Colors().Get("medium");
-	Color back = *GameData::Colors().Get("faint");
+	const Color &bright = *GameData::Colors().Get("bright");
+	const Color &dim = *GameData::Colors().Get("medium");
+	const Color &back = *GameData::Colors().Get("faint");
+	const Color &inactive = *GameData::Colors().Get("inactive");
 	if(canCancel)
 	{
 		string cancelText = isMission ? "Decline" : "Cancel";
@@ -154,40 +169,68 @@ void Dialog::Draw()
 	Point labelPos(
 		okPos.X() - .5 * font.Width(okText),
 		okPos.Y() - .5 * font.Height());
-	font.Draw(okText, labelPos, okIsActive ? bright : dim);
-	
+	font.Draw(okText, labelPos, isOkDisabled ? inactive : (okIsActive ? bright : dim));
+
 	// Draw the text.
 	text.Draw(textPos, dim);
-	
+
 	// Draw the input, if any.
 	if(!isMission && (intFun || stringFun))
 	{
 		FillShader::Fill(inputPos, Point(WIDTH - 20., 20.), back);
-		
+
 		Point stringPos(
 			inputPos.X() - (WIDTH - 20) * .5 + 5.,
 			inputPos.Y() - .5 * font.Height());
-		string truncated = font.TruncateFront(input, WIDTH - 30);
-		font.Draw(truncated, stringPos, bright);
-		
-		Point barPos(stringPos.X() + font.Width(truncated) + 2., inputPos.Y());
+		const auto inputText = DisplayText(input, {WIDTH - 30, Truncate::FRONT});
+		font.Draw(inputText, stringPos, bright);
+
+		Point barPos(stringPos.X() + font.FormattedWidth(inputText) + 2., inputPos.Y());
 		FillShader::Fill(barPos, Point(1., 16.), dim);
 	}
 }
 
 
 
-bool Dialog::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+// Format and add the text from the given node to the given string.
+void Dialog::ParseTextNode(const DataNode &node, size_t startingIndex, string &text)
+{
+	for(int i = startingIndex; i < node.Size(); ++i)
+	{
+		if(!text.empty())
+			text += "\n\t";
+		text += node.Token(i);
+	}
+	for(const DataNode &child : node)
+		for(int i = 0; i < child.Size(); ++i)
+		{
+			if(!text.empty())
+				text += "\n\t";
+			text += child.Token(i);
+		}
+}
+
+
+
+bool Dialog::AllowsFastForward() const noexcept
+{
+	return allowsFastForward;
+}
+
+
+
+bool Dialog::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	auto it = KEY_MAP.find(key);
-	if((it != KEY_MAP.end() || (key >= ' ' && key <= '~')) && !isMission && (intFun || stringFun))
+	bool isCloseRequest = key == SDLK_ESCAPE || (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI)));
+	if((it != KEY_MAP.end() || (key >= ' ' && key <= '~')) && !isMission && (intFun || stringFun) && !isCloseRequest)
 	{
 		int ascii = (it != KEY_MAP.end()) ? it->second : key;
 		char c = ((mod & KMOD_SHIFT) ? SHIFT[ascii] : ascii);
 		// Caps lock should shift letters, but not any other keys.
 		if((mod & KMOD_CAPS) && c >= 'a' && c <= 'z')
 			c += 'A' - 'a';
-		
+
 		if(stringFun)
 			input += c;
 		// Integer input should not allow leading zeros.
@@ -195,32 +238,53 @@ bool Dialog::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 			input += c;
 		else if(intFun && c >= '1' && c <= '9')
 			input += c;
+
+		if(validateFun)
+			isOkDisabled = !validateFun(input);
 	}
 	else if((key == SDLK_DELETE || key == SDLK_BACKSPACE) && !input.empty())
+	{
 		input.erase(input.length() - 1);
+		if(validateFun)
+			isOkDisabled = !validateFun(input);
+	}
 	else if(key == SDLK_TAB && canCancel)
 		okIsActive = !okIsActive;
 	else if(key == SDLK_LEFT)
 		okIsActive = !canCancel;
 	else if(key == SDLK_RIGHT)
 		okIsActive = true;
-	else if(key == SDLK_RETURN || key == SDLK_KP_ENTER || (isMission && (key == 'a' || key == 'd')))
+	else if(key == SDLK_RETURN || key == SDLK_KP_ENTER || isCloseRequest
+			|| (isMission && (key == 'a' || key == 'd')))
 	{
 		// Shortcuts for "accept" and "decline."
-		if(key == 'a')
+		if(key == 'a' || (!canCancel && isCloseRequest))
 			okIsActive = true;
-		if(key == 'd')
+		if(key == 'd' || (canCancel && isCloseRequest))
 			okIsActive = false;
-		if(okIsActive || isMission)
-			DoCallback();
-		
-		GetUI()->Pop(this);
+		if(boolFun)
+		{
+			DoCallback(okIsActive);
+			GetUI()->Pop(this);
+		}
+		else if(okIsActive || isMission)
+		{
+			// If the OK button is disabled (because the input failed the validation),
+			// don't execute the callback.
+			if(!isOkDisabled)
+			{
+				DoCallback();
+				GetUI()->Pop(this);
+			}
+		}
+		else
+			GetUI()->Pop(this);
 	}
 	else if((key == 'm' || command.Has(Command::MAP)) && system && player)
 		GetUI()->Push(new MapDetailPanel(*player, system));
 	else
 		return false;
-	
+
 	return true;
 }
 
@@ -229,14 +293,14 @@ bool Dialog::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 bool Dialog::Click(int x, int y, int clicks)
 {
 	Point clickPos(x, y);
-	
+
 	Point ok = clickPos - okPos;
 	if(fabs(ok.X()) < 40. && fabs(ok.Y()) < 20.)
 	{
 		okIsActive = true;
 		return DoKey(SDLK_RETURN);
 	}
-	
+
 	if(canCancel)
 	{
 		Point cancel = clickPos - cancelPos;
@@ -246,25 +310,28 @@ bool Dialog::Click(int x, int y, int clicks)
 			return DoKey(SDLK_RETURN);
 		}
 	}
-	
+
 	return true;
 }
 
 
 
 // Common code from all three constructors:
-void Dialog::Init(const string &message, bool canCancel, bool isMission)
+void Dialog::Init(const string &message, Truncate truncate, bool canCancel, bool isMission)
 {
+	SetInterruptible(isMission);
+
 	this->isMission = isMission;
 	this->canCancel = canCancel;
 	okIsActive = true;
-	
-	text.SetAlignment(WrappedText::JUSTIFIED);
+
+	text.SetAlignment(Alignment::JUSTIFIED);
 	text.SetWrapWidth(WIDTH - 20);
 	text.SetFont(FontSet::Get(14));
-	
+	text.SetTruncate(truncate);
+
 	text.Wrap(message);
-	
+
 	// The dialog with no extenders is 80 pixels tall. 10 pixels at the top and
 	// bottom are "padding," but text.Height() over-reports the height by about
 	// 5 pixels because it includes its own padding at the bottom. If there is a
@@ -279,22 +346,34 @@ void Dialog::Init(const string &message, bool canCancel, bool isMission)
 
 
 
-void Dialog::DoCallback() const
+void Dialog::DoCallback(const bool isOk) const
 {
 	if(isMission)
 	{
 		if(intFun)
 			intFun(okIsActive ? Conversation::ACCEPT : Conversation::DECLINE);
-		
+
 		return;
 	}
-	
+
 	if(intFun)
-		intFun(input.empty() ? 0 : stoi(input));
-	
+	{
+		// Only call the callback if the input can be converted to an int.
+		// Otherwise treat this as if the player clicked "cancel."
+		try {
+			intFun(stoi(input));
+		}
+		catch(...)
+		{
+		}
+	}
+
 	if(stringFun)
 		stringFun(input);
-	
+
 	if(voidFun)
 		voidFun();
+
+	if(boolFun)
+		boolFun(isOk);
 }
