@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "DrawList.h"
@@ -16,7 +19,6 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Preferences.h"
 #include "Screen.h"
 #include "Sprite.h"
-#include "SpriteSet.h"
 #include "SpriteShader.h"
 
 #include <cmath>
@@ -47,25 +49,19 @@ void DrawList::SetCenter(const Point &center, const Point &centerVelocity)
 // Add an object based on the Body class.
 bool DrawList::Add(const Body &body, double cloak)
 {
-	Point position = body.Position() - center;
-	Point blur = body.Velocity() - centerVelocity;
-	if(Cull(body, position, blur) || cloak >= 1.)
-		return false;
-	
-	Push(body, position, blur, cloak, 1., body.GetSwizzle());
-	return true;
+	return Add(body, body.Position(), cloak);
 }
 
 
 
-bool DrawList::Add(const Body &body, Point position)
+bool DrawList::Add(const Body &body, Point position, double cloak)
 {
 	position -= center;
 	Point blur = body.Velocity() - centerVelocity;
 	if(Cull(body, position, blur))
 		return false;
-	
-	Push(body, position, blur, 0., 1., body.GetSwizzle());
+
+	Push(body, std::move(position), std::move(blur), cloak, body.GetSwizzle());
 	return true;
 }
 
@@ -77,21 +73,8 @@ bool DrawList::AddUnblurred(const Body &body)
 	Point blur;
 	if(Cull(body, position, blur))
 		return false;
-	
-	Push(body, position, blur, 0., 1., body.GetSwizzle());
-	return true;
-}
 
-
-
-bool DrawList::AddProjectile(const Body &body, const Point &adjustedVelocity, double clip)
-{
-	Point position = body.Position() + .5 * body.Velocity() - center;
-	Point blur = adjustedVelocity - centerVelocity;
-	if(Cull(body, position, blur) || clip <= 0.)
-		return false;
-	
-	Push(body, position, blur, 0., clip, body.GetSwizzle());
+	Push(body, position, blur, 0., body.GetSwizzle());
 	return true;
 }
 
@@ -103,8 +86,8 @@ bool DrawList::AddSwizzled(const Body &body, int swizzle)
 	Point blur = body.Velocity() - centerVelocity;
 	if(Cull(body, position, blur))
 		return false;
-	
-	Push(body, position, blur, 0., 1., swizzle);
+
+	Push(body, position, blur, 0., swizzle);
 	return true;
 }
 
@@ -113,13 +96,11 @@ bool DrawList::AddSwizzled(const Body &body, int swizzle)
 // Draw all the items in this list.
 void DrawList::Draw() const
 {
-	bool showBlur = Preferences::Has("Render motion blur");
 	SpriteShader::Bind();
 
-	for(const Item &item : items)
-		SpriteShader::Add(
-			item.tex0, item.tex1, item.position, item.transform,
-			item.Swizzle(), item.Clip(), item.Fade(), showBlur ? item.blur : nullptr);
+	bool withBlur = Preferences::Has("Render motion blur");
+	for(const SpriteShader::Item &item : items)
+		SpriteShader::Add(item, withBlur);
 
 	SpriteShader::Unbind();
 }
@@ -130,7 +111,7 @@ bool DrawList::Cull(const Body &body, const Point &position, const Point &blur)
 {
 	if(!body.HasSprite() || !body.Zoom())
 		return true;
-	
+
 	Point unit = body.Facing().Unit();
 	// Cull sprites that are completely off screen, to reduce the number of draw
 	// calls that we issue (which may be the bottleneck on some systems).
@@ -143,39 +124,31 @@ bool DrawList::Cull(const Body &body, const Point &position, const Point &blur)
 		return true;
 	if(topLeft.X() > Screen::Right() || topLeft.Y() > Screen::Bottom())
 		return true;
-	
+
 	return false;
 }
 
 
 
-void DrawList::Push(const Body &body, Point pos, Point blur, double cloak, double clip, int swizzle)
+void DrawList::Push(const Body &body, Point pos, Point blur, double cloak, int swizzle)
 {
-	Item item;
-	
-	Body::Frame frame = body.GetFrame(step, isHighDPI);
-	item.tex0 = frame.first;
-	item.tex1 = frame.second;
-	item.flags = swizzle | (static_cast<uint32_t>(frame.fade * 256.f) << 8);
-	
+	SpriteShader::Item item;
+
+	item.texture = body.GetSprite()->Texture(isHighDPI);
+	item.swizzleMask = body.GetSprite()->SwizzleMask(isHighDPI);
+	item.frame = body.GetFrame(step);
+	item.frameCount = body.GetSprite()->Frames();
+
+	item.position[0] = static_cast<float>(pos.X() * zoom);
+	item.position[1] = static_cast<float>(pos.Y() * zoom);
+
 	// Get unit vectors in the direction of the object's width and height.
 	double width = body.Width();
 	double height = body.Height();
 	Point unit = body.Facing().Unit();
 	Point uw = unit * width;
 	Point uh = unit * height;
-	
-	item.clip = clip;
-	if(clip < 1.)
-	{
-		// "clip" is the fraction of its height that we're clipping the sprite
-		// to. We still want it to start at the same spot, though.
-		pos -= uh * ((1. - clip) * .5);
-		uh *= clip;
-	}
-	item.position[0] = static_cast<float>(pos.X() * zoom);
-	item.position[1] = static_cast<float>(pos.Y() * zoom);
-	
+
 	// (0, -1) means a zero-degree rotation (since negative Y is up).
 	uw *= zoom;
 	uh *= zoom;
@@ -183,45 +156,15 @@ void DrawList::Push(const Body &body, Point pos, Point blur, double cloak, doubl
 	item.transform[1] = uw.X();
 	item.transform[2] = -uh.X();
 	item.transform[3] = -uh.Y();
-	
+
 	// Calculate the blur vector, in texture coordinates.
 	blur *= zoom;
 	item.blur[0] = unit.Cross(blur) / (width * 4.);
 	item.blur[1] = -unit.Dot(blur) / (height * 4.);
 
-	if(cloak > 0.f)
-		item.Cloak(cloak);
-	
+	item.alpha = 1. - cloak;
+	item.swizzle = swizzle;
+	item.clip = 1.;
+
 	items.push_back(item);
 }
-
-
-		
-// Get the color swizzle.
-uint32_t DrawList::Item::Swizzle() const
-{
-	return (flags & 0xFF);
-}
-
-
-		
-float DrawList::Item::Clip() const
-{
-	return clip;
-}
-
-
-
-float DrawList::Item::Fade() const
-{
-	return (flags >> 8) / 256.f;
-}
-
-
-
-void DrawList::Item::Cloak(double cloak)
-{
-	tex1 = SpriteSet::Get("ship/cloaked")->Texture();
-	flags &= 0xFF;
-	flags |= static_cast<uint32_t>(cloak * 256.f) << 8;
-}
