@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef FLOTSAM_H_
@@ -21,6 +24,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <vector>
 
 class Effect;
+class Government;
 class Outfit;
 class Ship;
 class Visual;
@@ -30,17 +34,16 @@ class Visual;
 class Flotsam : public Body {
 public:
 	// Constructors for flotsam carrying either a commodity or an outfit.
-	Flotsam(const std::string &commodity, int count);
-	Flotsam(const Outfit *outfit, int count);
-	
-	/* Functions provided by the Body base class:
-	Frame GetFrame(int step = -1) const;
-	const Point &Position() const;
-	const Point &Velocity() const;
-	const Angle &Facing() const;
-	Point Unit() const;
-	*/
-	
+	Flotsam(const std::string &commodity, int count, const Government *sourceGovernment = nullptr);
+	Flotsam(const Outfit *outfit, int count, const Government *sourceGovernment = nullptr);
+
+	// Functions provided by the Body base class:
+	// Frame GetFrame(int step = -1) const;
+	// const Point &Position() const;
+	// const Point &Velocity() const;
+	// const Angle &Facing() const;
+	// Point Unit() const;
+
 	// Place this flotsam, and set the given ship as its source. This is a
 	// separate function because a ship may queue up flotsam to dump but take
 	// several frames before it finishes dumping it all.
@@ -49,12 +52,16 @@ public:
 	// the maximum relative velocity, or the exact relative velocity as a vector.
 	void Place(const Body &source, double maxVelocity = .5);
 	void Place(const Body &source, const Point &dv);
-	
+
 	// Move the object one time-step forward.
 	void Move(std::vector<Visual> &visuals);
-	
+
 	// This is the one ship that cannot pick up this flotsam.
 	const Ship *Source() const;
+	// Ships from this Government should not pick up this flotsam because it
+	// was explicitly dumped by a member of this government. (NPCs typically
+	// perform this type of dumping to appease pirates.)
+	const Government *SourceGovernment() const;
 	// This is what the flotsam contains:
 	const std::string &CommodityType() const;
 	const Outfit *OutfitType() const;
@@ -62,26 +69,27 @@ public:
 	// This is how big one "unit" of the flotsam is (in tons). If a ship has
 	// less than this amount of space, it can't pick up anything here.
 	double UnitSize() const;
-	
+
 	// Transfer contents to the collector ship. The flotsam velocity is
 	// stabilized in proportion to the amount being transferred.
 	int TransferTo(Ship *collector);
-	
-	
+
+
 public:
 	// Amount of tons that is expected per box.
 	static const int TONS_PER_BOX;
-	
-	
+
+
 private:
 	Angle spin;
 	int lifetime = 0;
 	double drag = 0.999;
-	
+
 	const Ship *source = nullptr;
 	std::string commodity;
 	const Outfit *outfit = nullptr;
 	int count = 0;
+	const Government *sourceGovernment = nullptr;
 };
 
 
