@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Galaxy.h"
@@ -23,9 +26,21 @@ void Galaxy::Load(const DataNode &node)
 {
 	for(const DataNode &child : node)
 	{
-		if(child.Token(0) == "pos" && child.Size() >= 3)
+		const bool remove = child.Token(0) == "remove";
+		const int keyIndex = remove;
+		const bool hasKey = child.Size() > keyIndex;
+		const string &key = hasKey ? child.Token(keyIndex) : child.Token(0);
+
+		if(remove && hasKey)
+		{
+			if(key == "sprite")
+				sprite = nullptr;
+			else
+				child.PrintTrace("Skipping unsupported use of \"remove\":");
+		}
+		else if(key == "pos" && child.Size() >= 3)
 			position = Point(child.Value(1), child.Value(2));
-		else if(child.Token(0) == "sprite" && child.Size() >= 2)
+		else if(key == "sprite" && child.Size() >= 2)
 			sprite = SpriteSet::Get(child.Token(1));
 		else
 			child.PrintTrace("Skipping unrecognized attribute:");
