@@ -17,7 +17,6 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 #include "DataWriter.h"
 #include "GameData.h"
-#include "Government.h"
 #include "Planet.h"
 #include "PlayerInfo.h"
 #include "System.h"
@@ -63,7 +62,7 @@ map<string, set<string>> GameEvent::DeferredDefinitions(const list<DataNode> &ch
 					definitions[key].emplace(name);
 			}
 			// Since this (or any other) event may be used to assign a planet to a system, we cannot
-			// do a robust "planet definition" check. Similarly, all other GameEvent-createable objects
+			// do a robust "planet definition" check. Similarly, all other GameEvent-creatable objects
 			// become valid once they appear as a root-level node that has at least one child node.
 			else
 				definitions[key].emplace(name);
@@ -219,16 +218,15 @@ void GameEvent::SetDate(const Date &date)
 
 
 
-void GameEvent::Apply(PlayerInfo &player)
+// Apply this event's changes to the player. Returns a list of data changes that need to
+// be applied in a batch with other events that are applied at the same time.
+list<DataNode> GameEvent::Apply(PlayerInfo &player)
 {
 	if(isDisabled)
-		return;
+		return {};
 
 	// Apply this event's ConditionSet to the player's conditions.
 	conditionsToApply.Apply(player.Conditions());
-	// Apply (and store a record of applying) this event's other general
-	// changes (e.g. updating an outfitter's inventory).
-	player.AddChanges(changes);
 
 	for(const System *system : systemsToUnvisit)
 		player.Unvisit(*system);
@@ -241,6 +239,10 @@ void GameEvent::Apply(PlayerInfo &player)
 		player.Visit(*system);
 	for(const Planet *planet : planetsToVisit)
 		player.Visit(*planet);
+
+	// Return this event's data changes so that they can be batch applied
+	// with the changes from other events.
+	return std::move(changes);
 }
 
 
