@@ -78,34 +78,6 @@ namespace {
 
 
 
-Government::RaidFleet::RaidFleet(const Fleet *fleet, double minAttraction, double maxAttraction)
-	: fleet(fleet), minAttraction(minAttraction), maxAttraction(maxAttraction)
-{
-}
-
-
-
-const Fleet *Government::RaidFleet::GetFleet() const
-{
-	return fleet;
-}
-
-
-
-double Government::RaidFleet::MinAttraction() const
-{
-	return minAttraction;
-}
-
-
-
-double Government::RaidFleet::MaxAttraction() const
-{
-	return maxAttraction;
-}
-
-
-
 // Default constructor.
 Government::Government()
 {
@@ -165,6 +137,8 @@ void Government::Load(const DataNode &node)
 		{
 			if(key == "provoked on scan")
 				provokedOnScan = false;
+			else if(key == "travel restrictions")
+				travelRestrictions = LocationFilter{};
 			else if(key == "reputation")
 			{
 				for(const DataNode &grand : child)
@@ -203,9 +177,15 @@ void Government::Load(const DataNode &node)
 			else if(key == "foreign penalties for")
 				useForeignPenaltiesFor.clear();
 			else if(key == "illegals")
-				illegals.clear();
+			{
+				illegalOutfits.clear();
+				illegalShips.clear();
+			}
 			else if(key == "atrocities")
-				atrocities.clear();
+			{
+				atrocityOutfits.clear();
+				atrocityShips.clear();
+			}
 			else
 				child.PrintTrace("Cannot \"remove\" the given key:");
 
@@ -217,21 +197,7 @@ void Government::Load(const DataNode &node)
 		}
 
 		if(key == "raid")
-		{
-			const Fleet *fleet = GameData::Fleets().Get(child.Token(valueIndex));
-			if(remove)
-			{
-				for(auto it = raidFleets.begin(); it != raidFleets.end(); )
-					if(it->GetFleet() == fleet)
-						it = raidFleets.erase(it);
-					else
-						++it;
-			}
-			else
-				raidFleets.emplace_back(fleet,
-					child.Size() > (valueIndex + 1) ? child.Value(valueIndex + 1) : 2.,
-					child.Size() > (valueIndex + 2) ? child.Value(valueIndex + 2) : 0.);
-		}
+			RaidFleet::Load(raidFleets, child, remove, valueIndex);
 		// Handle the attributes which cannot have a value removed.
 		else if(remove)
 			child.PrintTrace("Cannot \"remove\" a specific value from the given key:");
@@ -310,42 +276,80 @@ void Government::Load(const DataNode &node)
 		else if(key == "illegals")
 		{
 			if(!add)
-				illegals.clear();
+			{
+				illegalOutfits.clear();
+				illegalShips.clear();
+			}
 			for(const DataNode &grand : child)
 				if(grand.Size() >= 2)
 				{
-					if(grand.Token(0) == "ignore")
-						illegals[GameData::Outfits().Get(grand.Token(1))] = 0;
+					if(grand.Token(0) == "remove")
+					{
+						if(grand.Size() >= 3 && grand.Token(1) == "ship")
+						{
+							if(!illegalShips.erase(grand.Token(2)))
+								grand.PrintTrace("Invalid remove, ship not found in existing illegals:");
+						}
+						else if(!illegalOutfits.erase(GameData::Outfits().Get(grand.Token(1))))
+							grand.PrintTrace("Invalid remove, outfit not found in existing illegals:");
+					}
+					else if(grand.Token(0) == "ignore")
+					{
+						if(grand.Size() >= 3 && grand.Token(1) == "ship")
+							illegalShips[grand.Token(2)] = 0;
+						else
+							illegalOutfits[GameData::Outfits().Get(grand.Token(1))] = 0;
+					}
+					else if(grand.Size() >= 3 && grand.Token(0) == "ship")
+						illegalShips[grand.Token(1)] = grand.Value(2);
 					else
-						illegals[GameData::Outfits().Get(grand.Token(0))] = grand.Value(1);
+						illegalOutfits[GameData::Outfits().Get(grand.Token(0))] = grand.Value(1);
 				}
-				else if(grand.Size() >= 3 && grand.Token(0) == "remove")
-				{
-					if(!illegals.erase(GameData::Outfits().Get(grand.Token(1))))
-						grand.PrintTrace("Invalid remove, outfit not found in existing illegals:");
-				}
-				else
-					grand.PrintTrace("Skipping unrecognized attribute:");
 		}
 		else if(key == "atrocities")
 		{
 			if(!add)
-				atrocities.clear();
+			{
+				atrocityOutfits.clear();
+				atrocityShips.clear();
+			}
 			for(const DataNode &grand : child)
 				if(grand.Size() >= 2)
 				{
-					if(grand.Token(0) == "remove" && !atrocities.erase(GameData::Outfits().Get(grand.Token(1))))
-						grand.PrintTrace("Invalid remove, outfit not found in existing atrocities:");
+					if(grand.Token(0) == "remove")
+					{
+						if(grand.Size() >= 3 && grand.Token(1) == "ship")
+						{
+							if(!atrocityShips.erase(grand.Token(2)))
+								grand.PrintTrace("Invalid remove, ship not found in existing atrocities:");
+						}
+						else if(!atrocityOutfits.erase(GameData::Outfits().Get(grand.Token(1))))
+							grand.PrintTrace("Invalid remove, outfit not found in existing atrocities:");
+					}
 					else if(grand.Token(0) == "ignore")
-						atrocities[GameData::Outfits().Get(grand.Token(1))] = false;
+					{
+						if(grand.Size() >= 3 && grand.Token(1) == "ship")
+							atrocityShips[grand.Token(2)] = false;
+						else
+							atrocityOutfits[GameData::Outfits().Get(grand.Token(1))] = false;
+					}
+					else if(grand.Token(0) == "ship")
+						atrocityShips[grand.Token(1)] = true;
 				}
 				else
-					atrocities[GameData::Outfits().Get(grand.Token(0))] = true;
+					atrocityOutfits[GameData::Outfits().Get(grand.Token(0))] = true;
 		}
 		else if(key == "enforces" && child.HasChildren())
 			enforcementZones.emplace_back(child);
 		else if(key == "provoked on scan")
 			provokedOnScan = true;
+		else if(key == "travel restrictions" && child.HasChildren())
+		{
+			if(add)
+				travelRestrictions.Load(child);
+			else
+				travelRestrictions = LocationFilter(child);
+		}
 		else if(key == "foreign penalties for")
 			for(const DataNode &grand : child)
 				useForeignPenaltiesFor.insert(GameData::Governments().Get(grand.Token(0))->id);
@@ -591,7 +595,7 @@ bool Government::SendUntranslatedHails() const
 // Pirate raids in this government's systems use these fleet definitions. If
 // it is empty, there are no pirate raids.
 // The second attribute denotes the minimal and maximal attraction required for the fleet to appear.
-const vector<Government::RaidFleet> &Government::RaidFleets() const
+const vector<RaidFleet> &Government::RaidFleets() const
 {
 	return raidFleets;
 }
@@ -653,20 +657,29 @@ string Government::Fine(PlayerInfo &player, int scan, const Ship *target, double
 
 bool Government::Condemns(const Outfit *outfit) const
 {
-	const auto isAtrocity = atrocities.find(outfit);
-	bool found = isAtrocity != atrocities.cend();
+	const auto isAtrocity = atrocityOutfits.find(outfit);
+	bool found = isAtrocity != atrocityOutfits.cend();
 	return (found && isAtrocity->second) || (!found && outfit->Get("atrocity") > 0.);
 }
 
 
 
+bool Government::Condemns(const Ship *ship) const
+{
+	const auto isAtrocity = atrocityShips.find(ship->TrueModelName());
+	bool found = isAtrocity != atrocityShips.cend();
+	return (found && isAtrocity->second) || (!found && ship->BaseAttributes().Get("atrocity") > 0.);
+}
+
+
+
 int Government::Fines(const Outfit *outfit) const
 {
 	// If this government doesn't fine anything it won't fine this outfit.
 	if(!fine)
 		return 0;
 
-	for(const auto &it : illegals)
+	for(const auto &it : illegalOutfits)
 		if(it.first == outfit)
 			return it.second;
 	return outfit->Get("illegal");
@@ -674,6 +687,20 @@ int Government::Fines(const Outfit *outfit) const
 
 
 
+int Government::Fines(const Ship *ship) const
+{
+	// If this government doesn't fine anything it won't fine this ship.
+	if(!fine)
+		return 0;
+
+	for(const auto &it : illegalShips)
+		if(it.first == ship->TrueModelName())
+			return it.second;
+	return ship->BaseAttributes().Get("illegal");
+}
+
+
+
 bool Government::FinesContents(const Ship *ship) const
 {
 	for(auto &it : ship->Outfits())
@@ -739,3 +766,17 @@ bool Government::IsProvokedOnScan() const
 {
 	return provokedOnScan;
 }
+
+
+
+bool Government::IsRestrictedFrom(const System &system) const
+{
+	return !travelRestrictions.IsEmpty() && travelRestrictions.Matches(&system);
+}
+
+
+
+bool Government::IsRestrictedFrom(const Planet &planet) const
+{
+	return !travelRestrictions.IsEmpty() && travelRestrictions.Matches(&planet);
+}
