@@ -19,6 +19,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "Color.h"
 #include "ExclusiveItem.h"
 #include "LocationFilter.h"
+#include "RaidFleet.h"
 
 #include <limits>
 #include <map>
@@ -46,21 +47,6 @@ class System;
 // color "swizzle." Some government's ships can also be easier or harder to
 // bribe than others.
 class Government {
-public:
-	class RaidFleet {
-		public:
-			RaidFleet(const Fleet *fleet, double minAttraction, double maxAttraction);
-			const Fleet *GetFleet() const;
-			double MinAttraction() const;
-			double MaxAttraction() const;
-
-		private:
-			const Fleet *fleet = nullptr;
-			double minAttraction;
-			double maxAttraction;
-	};
-
-
 public:
 	// Default constructor.
 	Government();
@@ -136,8 +122,10 @@ public:
 	std::string Fine(PlayerInfo &player, int scan = 0, const Ship *target = nullptr, double security = 1.) const;
 	// Check to see if the items are condemnable (atrocities) or warrant a fine.
 	bool Condemns(const Outfit *outfit) const;
-	// Returns the fine for given outfit for this government.
+	bool Condemns(const Ship *ship) const;
+	// Returns the fine for given item for this government.
 	int Fines(const Outfit *outfit) const;
+	int Fines(const Ship *ship) const;
 	// Check if given ship has illegal outfits or cargo.
 	bool FinesContents(const Ship *ship) const;
 
@@ -154,6 +142,10 @@ public:
 
 	bool IsProvokedOnScan() const;
 
+	// Determine if ships from this government can travel to the given system or planet.
+	bool IsRestrictedFrom(const System &system) const;
+	bool IsRestrictedFrom(const Planet &planet) const;
+
 
 private:
 	unsigned id;
@@ -169,11 +161,14 @@ private:
 	double reputationMax = std::numeric_limits<double>::max();
 	double reputationMin = std::numeric_limits<double>::lowest();
 	std::map<int, double> penaltyFor;
-	std::map<const Outfit*, int> illegals;
-	std::map<const Outfit*, bool> atrocities;
+	std::map<const Outfit*, int> illegalOutfits;
+	std::map<std::string, int> illegalShips;
+	std::map<const Outfit*, bool> atrocityOutfits;
+	std::map<std::string, bool> atrocityShips;
 	double bribe = 0.;
 	double fine = 1.;
 	std::vector<LocationFilter> enforcementZones;
+	LocationFilter travelRestrictions;
 	const Conversation *deathSentence = nullptr;
 	const Phrase *friendlyHail = nullptr;
 	const Phrase *friendlyDisabledHail = nullptr;
