@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef HAIL_PANEL_H_
@@ -22,6 +25,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <memory>
 #include <string>
 
+class Government;
 class Planet;
 class PlayerInfo;
 class Ship;
@@ -35,36 +39,41 @@ class StellarObject;
 // to bribe a planet to allow you to land there.
 class HailPanel : public Panel {
 public:
-	HailPanel(PlayerInfo &player, const std::shared_ptr<Ship> &ship);
+	HailPanel(PlayerInfo &player, const std::shared_ptr<Ship> &ship,
+		std::function<void(const Government *)> bribeCallback);
 	HailPanel(PlayerInfo &player, const StellarObject *object);
-	
+
 	virtual void Draw() override;
-	
-	
+
+
 protected:
 	// Only override the ones you need; the default action is to return false.
-	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command) override;
-	
-	
+	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress) override;
+
+
 private:
 	void SetBribe(double scale);
-	
-	
+
+
 private:
 	PlayerInfo &player;
 	std::shared_ptr<Ship> ship = nullptr;
+	std::function<void(const Government *)> bribeCallback = nullptr;
 	const Planet *planet = nullptr;
 	const Sprite *sprite = nullptr;
 	Angle facing;
-	
+
 	std::string header;
 	std::string message;
-	
+
 	int64_t bribe = 0;
+	const Government *bribed = nullptr;
 	bool playerNeedsHelp = false;
+	bool canAssistPlayer = true;
 	bool canGiveFuel = false;
 	bool canRepair = false;
 	bool hasLanguage = true;
+	bool requestedToBribeShip = false;
 };
 
 
