@@ -15,6 +15,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 #include "Interface.h"
 
+#include "Angle.h"
 #include "DataNode.h"
 #include "text/DisplayText.h"
 #include "FillShader.h"
@@ -26,6 +27,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "LineShader.h"
 #include "OutlineShader.h"
 #include "Panel.h"
+#include "PointerShader.h"
 #include "Rectangle.h"
 #include "RingShader.h"
 #include "Screen.h"
@@ -119,6 +121,8 @@ void Interface::Load(const DataNode &node)
 				elements.push_back(new TextElement(child, anchor));
 			else if(child.Token(0) == "bar" || child.Token(0) == "ring")
 				elements.push_back(new BarElement(child, anchor));
+			else if(child.Token(0) == "pointer")
+				elements.push_back(new PointerElement(child, anchor));
 			else if(child.Token(0) == "line")
 				elements.push_back(new LineElement(child, anchor));
 			else
@@ -194,6 +198,14 @@ Point Interface::AnchoredPoint::Get() const
 
 
 
+Point Interface::AnchoredPoint::Get(const Information &info) const
+{
+	const Rectangle &region = info.GetCustomRegion();
+	return position + region.Center() + .5 * region.Dimensions() * anchor;
+}
+
+
+
 void Interface::AnchoredPoint::Set(const Point &position, const Point &anchor)
 {
 	this->position = position;
@@ -308,7 +320,7 @@ void Interface::Element::Draw(const Information &info, Panel *panel) const
 		return;
 
 	// Get the bounding box of this element, relative to the anchor point.
-	Rectangle box = Bounds();
+	Rectangle box = (info.HasCustomRegion() ? Bounds(info) : Bounds());
 	// Check if this element is active.
 	int state = info.HasCondition(activeIf);
 	// Check if the mouse is hovering over this element.
@@ -346,6 +358,13 @@ Rectangle Interface::Element::Bounds() const
 
 
 
+Rectangle Interface::Element::Bounds(const Information &info) const
+{
+	return Rectangle::WithCorners(from.Get(info), to.Get(info));
+}
+
+
+
 // Parse the given data line: one that is not recognized by Element
 // itself. This returns false if it does not recognize the line, either.
 bool Interface::Element::ParseLine(const DataNode &node)
@@ -632,6 +651,8 @@ bool Interface::BarElement::ParseLine(const DataNode &node)
 		spanAngle = max(0., min(360., node.Value(1)));
 	else if(node.Token(0) == "start angle" && node.Size() >= 2)
 		startAngle = max(0., min(360., node.Value(1)));
+	else if(node.Token(0) == "reversed")
+		reversed = true;
 	else
 		return false;
 
@@ -665,9 +686,12 @@ void Interface::BarElement::Draw(const Rectangle &rect, const Information &info,
 	else
 	{
 		// Figue out where the line should be drawn from and to.
-		// Note: this assumes that the bottom of the rectangle is the start.
-		Point start = rect.BottomRight();
+		// Note: the default start position is the bottom right.
+		// If "reversed" was specified, the top left will be used instead.
+		Point start = reversed ? rect.TopLeft() : rect.BottomRight();
 		Point dimensions = -rect.Dimensions();
+		if(reversed)
+			dimensions *= -1.;
 		double length = dimensions.Length();
 
 		// We will have (segments - 1) gaps between the segments.
@@ -690,6 +714,59 @@ void Interface::BarElement::Draw(const Rectangle &rect, const Information &info,
 
 
 
+
+// Members of the PointerElement class:
+
+// Constructor.
+Interface::PointerElement::PointerElement(const DataNode &node, const Point &globalAnchor)
+{
+	Load(node, globalAnchor);
+
+	// Fill in a default color if none is specified.
+	if(!color)
+		color = GameData::Colors().Get("medium");
+
+	// Set a default orientation if none is specified.
+	if(!orientation)
+		orientation = Point(0., -1.);
+}
+
+
+
+// Parse the given data line: one that is not recognized by Element
+// itself. This returns false if it does not recognize the line, either.
+bool Interface::PointerElement::ParseLine(const DataNode &node)
+{
+	if(node.Token(0) == "color" && node.Size() >= 2)
+		color = GameData::Colors().Get(node.Token(1));
+	else if(node.Token(0) == "orientation angle" && node.Size() >= 2)
+	{
+		const Angle direction(node.Value(1));
+		orientation = direction.Unit();
+	}
+	else if(node.Token(0) == "orientation vector" && node.Size() >= 3)
+	{
+		orientation.X() = node.Value(1);
+		orientation.Y() = node.Value(2);
+	}
+	else
+		return false;
+
+	return true;
+}
+
+
+
+void Interface::PointerElement::Draw(const Rectangle &rect, const Information &info, int state) const
+{
+	const Point center = rect.Center();
+	const float width = rect.Width();
+	const float height = rect.Height();
+	PointerShader::Draw(center, orientation, width, height, 0.f, *color);
+}
+
+
+
 // Members of the LineElement class:
 
 // Constructor.
