@@ -7,11 +7,16 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "LocationFilter.h"
 
+#include "CategoryList.h"
+#include "CategoryTypes.h"
 #include "DataNode.h"
 #include "DataWriter.h"
 #include "DistanceMap.h"
@@ -64,30 +69,44 @@ namespace {
 		}
 		return false;
 	}
-	
+
 	// Check if the given system is within the given distance of the center.
-	int Distance(const System *center, const System *system, int maximum)
+	int Distance(const System *center, const System *system, int maximum, DistanceCalculationSettings distanceSettings)
 	{
 		// This function should only ever be called from the main thread, but
 		// just to be sure, use mutex protection on the static locals.
 		static mutex distanceMutex;
 		lock_guard<mutex> lock(distanceMutex);
-		
+
 		static const System *previousCenter = center;
-		static DistanceMap distance(center, -1, maximum);
+		static DistanceMap distance(
+			center,
+			distanceSettings.WormholeStrat(),
+			distanceSettings.AssumesJumpDrive(),
+			-1,
+			maximum
+		);
 		static int previousMaximum = maximum;
-		
-		if(center != previousCenter || maximum > previousMaximum)
+		static DistanceCalculationSettings previousDistanceSettings = distanceSettings;
+
+		if(center != previousCenter || maximum > previousMaximum || distanceSettings != previousDistanceSettings)
 		{
 			previousCenter = center;
 			previousMaximum = maximum;
-			distance = DistanceMap(center, -1, maximum);
+			previousDistanceSettings = distanceSettings;
+			distance = DistanceMap(
+				center,
+				distanceSettings.WormholeStrat(),
+				distanceSettings.AssumesJumpDrive(),
+				-1,
+				maximum
+			);
 		}
 		// If the distance is greater than the maximum, this is not a match.
 		int d = distance.Days(system);
 		return (d > maximum) ? -1 : d;
 	}
-	
+
 	// Check that at least one neighbor of the hub system matches, for each of the neighbor filters.
 	// False if at least one filter fails to match, true if all filters find at least one match.
 	bool MatchesNeighborFilters(const list<LocationFilter> &neighborFilters, const System *hub, const System *origin)
@@ -106,7 +125,7 @@ namespace {
 		}
 		return true;
 	}
-	
+
 	// Validity check for this filter's sets. Only one element must be valid.
 	template <class T>
 	bool CheckValidity(const set<const T *> &c)
@@ -129,12 +148,12 @@ namespace {
 	{
 		if(l.empty())
 			return true;
-		
+
 		for(auto &&outfits : l)
 			for(auto &&outfit : outfits)
 				if(outfit->IsDefined())
 					return true;
-		
+
 		return false;
 	}
 }
@@ -169,6 +188,10 @@ void LocationFilter::Load(const DataNode &node)
 		else
 			LoadChild(child);
 	}
+
+	isEmpty = planets.empty() && attributes.empty() && systems.empty() && governments.empty()
+		&& !center && originMaxDistance < 0 && notFilters.empty() && neighborFilters.empty()
+		&& outfits.empty() && shipCategory.empty();
 }
 
 
@@ -233,7 +256,7 @@ void LocationFilter::Save(DataWriter &out) const
 			out.BeginChild();
 			{
 				for(const Outfit *outfit : it)
-					out.Write(outfit->Name());
+					out.Write(outfit->TrueName());
 			}
 			out.EndChild();
 		}
@@ -258,9 +281,7 @@ void LocationFilter::Save(DataWriter &out) const
 // Check if this filter contains any specifications.
 bool LocationFilter::IsEmpty() const
 {
-	return planets.empty() && attributes.empty() && systems.empty() && governments.empty()
-		&& !center && originMaxDistance < 0 && notFilters.empty() && neighborFilters.empty()
-		&& outfits.empty() && shipCategory.empty();
+	return isEmpty;
 }
 
 
@@ -272,38 +293,40 @@ bool LocationFilter::IsValid() const
 {
 	if(IsEmpty())
 		return true;
-	
+
 	if(!CheckValidity(planets))
 		return false;
-	
+
 	// Attributes are always considered valid.
-	
+
 	if(!CheckValidity(systems))
 		return false;
-	
+
 	// Governments are always considered valid.
-	
+
 	// The "center" of a "near <system>" filter must be valid.
 	if(center && !center->IsValid())
 		return false;
-	
+
 	if(!CheckValidity(outfits))
 		return false;
-	
+
 	if(!shipCategory.empty())
 	{
 		// At least one desired category must be valid.
-		auto categories = set<string>(Ship::CATEGORIES.begin(), Ship::CATEGORIES.end());
-		if(!SetsIntersect(shipCategory, categories))
+		set<string> categoriesSet;
+		for(const auto &category : GameData::GetCategory(CategoryType::SHIP))
+			categoriesSet.insert(category.Name());
+		if(!SetsIntersect(shipCategory, categoriesSet))
 			return false;
 	}
-	
+
 	if(!CheckValidity(notFilters))
 		return false;
-	
+
 	if(!CheckValidity(neighborFilters))
 		return false;
-	
+
 	return true;
 }
 
@@ -314,29 +337,29 @@ bool LocationFilter::Matches(const Planet *planet, const System *origin) const
 {
 	if(!planet || !planet->IsValid())
 		return false;
-	
+
 	// If a ship class was given, do not match planets.
 	if(!shipCategory.empty())
 		return false;
-	
+
 	if(!governments.empty() && !governments.count(planet->GetGovernment()))
 		return false;
-	
+
 	if(!planets.empty() && !planets.count(planet))
 		return false;
 	for(const set<string> &attr : attributes)
 		if(!SetsIntersect(attr, planet->Attributes()))
 			return false;
-	
+
 	for(const LocationFilter &filter : notFilters)
 		if(filter.Matches(planet, origin))
 			return false;
-	
+
 	// If outfits are specified, make sure they can be bought here.
 	for(const set<const Outfit *> &outfitList : outfits)
 		if(!SetsIntersect(outfitList, planet->Outfitter()))
 			return false;
-	
+
 	return Matches(planet->GetSystem(), origin, true);
 }
 
@@ -347,7 +370,7 @@ bool LocationFilter::Matches(const System *system, const System *origin) const
 	// If a ship class was given, do not match systems.
 	if(!shipCategory.empty())
 		return false;
-	
+
 	return Matches(system, origin, false);
 }
 
@@ -362,10 +385,10 @@ bool LocationFilter::Matches(const Ship &ship) const
 		return false;
 	if(!governments.empty() && !governments.count(ship.GetGovernment()))
 		return false;
-	
+
 	if(!shipCategory.empty() && !shipCategory.count(ship.Attributes().Category()))
 		return false;
-	
+
 	if(!attributes.empty())
 	{
 		// Create a set from the positive-valued attributes of this ship.
@@ -377,7 +400,7 @@ bool LocationFilter::Matches(const Ship &ship) const
 			if(!SetsIntersect(attr, shipAttributes))
 				return false;
 	}
-	
+
 	if(!outfits.empty())
 	{
 		// Create a set from all installed and carried outfits.
@@ -392,19 +415,19 @@ bool LocationFilter::Matches(const Ship &ship) const
 			if(!SetsIntersect(outfitSet, shipOutfits))
 				return false;
 	}
-	
+
 	for(const LocationFilter &filter : notFilters)
 		if(filter.Matches(ship))
 			return false;
-	
+
 	if(!MatchesNeighborFilters(neighborFilters, origin, origin))
 		return false;
-	
+
 	// Check if this ship's current system meets a "near <system>" criterion.
 	// (Ships only offer missions, so no "distance" criteria need to be checked.)
-	if(center && Distance(center, origin, centerMaxDistance) < centerMinDistance)
+	if(center && Distance(center, origin, centerMaxDistance, centerDistanceOptions) < centerMinDistance)
 		return false;
-	
+
 	return true;
 }
 
@@ -416,22 +439,24 @@ LocationFilter LocationFilter::SetOrigin(const System *origin) const
 	// If there is no distance filter, then no conversion is needed.
 	if(IsEmpty() || originMaxDistance < 0)
 		return *this;
-	
+
 	// If the system is invalid, or a "near <system>" filter already
 	// exists, do not convert "distance" to "near".
 	if(!origin || center)
 		return *this;
-	
+
 	// Copy all parts of this instantiated filter into the result.
 	LocationFilter result = *this;
 	// Perform the conversion.
 	result.center = origin;
 	result.centerMinDistance = originMinDistance;
 	result.centerMaxDistance = originMaxDistance;
+	result.centerDistanceOptions = originDistanceOptions;
 	// Revert "distance" parameters to their default.
 	result.originMinDistance = 0;
 	result.originMaxDistance = -1;
-	
+	result.originDistanceOptions = DistanceCalculationSettings{};
+
 	return result;
 }
 
@@ -444,11 +469,12 @@ const System *LocationFilter::PickSystem(const System *origin) const
 	vector<const System *> options;
 	for(const auto &it : GameData::Systems())
 	{
-		// Skip entries with incomplete data.
-		if(!it.second.IsValid())
+		const System &system = it.second;
+		// Skip systems with incomplete data or that are inaccessible.
+		if(!system.IsValid() || system.Inaccessible())
 			continue;
-		if(Matches(&it.second, origin))
-			options.push_back(&it.second);
+		if(Matches(&system, origin))
+			options.push_back(&system);
 	}
 	return options.empty() ? nullptr : options[Random::Int(options.size())];
 }
@@ -463,8 +489,8 @@ const Planet *LocationFilter::PickPlanet(const System *origin, bool hasClearance
 	for(const auto &it : GameData::Planets())
 	{
 		const Planet &planet = it.second;
-		// Skip entries with incomplete data.
-		if(!planet.IsValid())
+		// Skip planets with incomplete data or which are from inaccessible systems.
+		if(!planet.IsValid() || (planet.GetSystem() && planet.GetSystem()->Inaccessible()))
 			continue;
 		// Skip planets that do not offer special jobs or missions, unless they were explicitly listed as options.
 		if(planet.IsWormhole() || (requireSpaceport && !planet.HasSpaceport()) || (!hasClearance && !planet.CanLand()))
@@ -485,7 +511,8 @@ void LocationFilter::LoadChild(const DataNode &child)
 	int valueIndex = 1 + isNot;
 	const string &key = child.Token(valueIndex - 1);
 	if(key == "not" || key == "neighbor")
-		child.PrintTrace("Skipping unsupported use of 'not' and 'neighbor'. These keywords must be nested if used together.");
+		child.PrintTrace("Error: Skipping unsupported use of 'not' and 'neighbor'."
+			" These keywords must be nested if used together.");
 	else if(key == "planet")
 	{
 		for(int i = valueIndex; i < child.Size(); ++i)
@@ -532,6 +559,9 @@ void LocationFilter::LoadChild(const DataNode &child)
 			centerMinDistance = child.Value(1 + valueIndex);
 			centerMaxDistance = child.Value(2 + valueIndex);
 		}
+
+		if(child.HasChildren())
+			centerDistanceOptions.Load(child);
 	}
 	else if(key == "distance" && child.Size() >= 1 + valueIndex)
 	{
@@ -542,6 +572,9 @@ void LocationFilter::LoadChild(const DataNode &child)
 			originMinDistance = child.Value(valueIndex);
 			originMaxDistance = child.Value(1 + valueIndex);
 		}
+
+		if(child.HasChildren())
+			originDistanceOptions.Load(child);
 	}
 	else if(key == "category" && child.Size() >= 2 + isNot)
 	{
@@ -564,7 +597,7 @@ void LocationFilter::LoadChild(const DataNode &child)
 			outfits.pop_back();
 	}
 	else
-		child.PrintTrace("Unrecognized location filter:");
+		child.PrintTrace("Skipping unrecognized attribute:");
 }
 
 
@@ -575,14 +608,14 @@ bool LocationFilter::Matches(const System *system, const System *origin, bool di
 		return false;
 	if(!systems.empty() && !systems.count(system))
 		return false;
-	
+
 	// Don't check these filters again if they were already checked as a part of
 	// checking if a planet matches.
 	if(!didPlanet)
 	{
 		if(!governments.empty() && !governments.count(system->GetGovernment()))
 			return false;
-		
+
 		// This filter is being applied to a system, not a planet.
 		// Check whether the system, or any planet within it, has one of the
 		// required attributes from each set.
@@ -594,26 +627,26 @@ bool LocationFilter::Matches(const System *system, const System *origin, bool di
 				for(const StellarObject &object : system->Objects())
 					if(object.HasSprite() && object.HasValidPlanet())
 						matches |= SetsIntersect(attr, object.GetPlanet()->Attributes());
-				
+
 				if(!matches)
 					return false;
 			}
 		}
-		
+
 		for(const LocationFilter &filter : notFilters)
 			if(filter.Matches(system, origin))
 				return false;
 	}
-	
+
 	if(!MatchesNeighborFilters(neighborFilters, system, origin))
 		return false;
-	
+
 	// Check this system's distance from the desired reference system.
-	if(center && Distance(center, system, centerMaxDistance) < centerMinDistance)
+	if(center && Distance(center, system, centerMaxDistance, centerDistanceOptions) < centerMinDistance)
 		return false;
 	if(origin && originMaxDistance >= 0
-			&& Distance(origin, system, originMaxDistance) < originMinDistance)
+			&& Distance(origin, system, originMaxDistance, originDistanceOptions) < originMinDistance)
 		return false;
-	
+
 	return true;
 }
