@@ -7,24 +7,32 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "LogbookPanel.h"
 
+#include "text/alignment.hpp"
 #include "Color.h"
+#include "text/DisplayText.h"
 #include "FillShader.h"
-#include "Font.h"
-#include "FontSet.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
 #include "GameData.h"
+#include "text/layout.hpp"
+#include "PlayerInfo.h"
 #include "Preferences.h"
 #include "Screen.h"
 #include "Sprite.h"
 #include "SpriteSet.h"
 #include "SpriteShader.h"
 #include "UI.h"
-#include "WrappedText.h"
+#include "text/WrappedText.h"
 
+#include <algorithm>
 #include <set>
 
 using namespace std;
@@ -35,8 +43,12 @@ namespace {
 	const double PAD = 10.;
 	const double WIDTH = SIDEBAR_WIDTH + TEXT_WIDTH;
 	const double LINE_HEIGHT = 25.;
+
+	// The minimum distance in pixels between the selected month and the edge of the screen before the month gets centered
+	const double MINIMUM_SELECTION_DISTANCE = LINE_HEIGHT * 3;
+
 	const double GAP = 30.;
-	static const string MONTH[] = {
+	const string MONTH[] = {
 		"  January", "  February", "  March", "  April", "  May", "  June",
 		"  July", "  August", "  September", "  October", "  November", "  December"};
 }
@@ -62,24 +74,24 @@ void LogbookPanel::Draw()
 {
 	// Dim out everything outside this panel.
 	DrawBackdrop();
-	
+
 	// Draw the panel. The sidebar should be slightly darker than the rest.
-	Color sideColor(.09, 1.);
+	const Color &sideColor = *GameData::Colors().Get("logbook sidebar");
 	FillShader::Fill(
 		Point(Screen::Left() + .5 * SIDEBAR_WIDTH, 0.),
 		Point(SIDEBAR_WIDTH, Screen::Height()),
 		sideColor);
-	Color backColor(.125, 1.);
+	const Color &backColor = *GameData::Colors().Get("logbook background");
 	FillShader::Fill(
 		Point(Screen::Left() + SIDEBAR_WIDTH + .5 * TEXT_WIDTH, 0.),
 		Point(TEXT_WIDTH, Screen::Height()),
 		backColor);
-	Color lineColor(.2, 1.);
+	const Color &lineColor = *GameData::Colors().Get("logbook line");
 	FillShader::Fill(
 		Point(Screen::Left() + SIDEBAR_WIDTH - .5, 0.),
 		Point(1., Screen::Height()),
 		lineColor);
-	
+
 	const Sprite *edgeSprite = SpriteSet::Get("ui/right edge");
 	if(edgeSprite->Height())
 	{
@@ -91,13 +103,13 @@ void LogbookPanel::Draw()
 		for( ; pos.Y() - .5 * spriteHeight < Screen::Bottom(); pos.Y() += spriteHeight)
 			SpriteShader::Draw(edgeSprite, pos);
 	}
-	
+
 	// Colors to be used for drawing the log.
 	const Font &font = FontSet::Get(14);
-	Color dim = *GameData::Colors().Get("dim");
-	Color medium = *GameData::Colors().Get("medium");
-	Color bright = *GameData::Colors().Get("bright");
-	
+	const Color &dim = *GameData::Colors().Get("dim");
+	const Color &medium = *GameData::Colors().Get("medium");
+	const Color &bright = *GameData::Colors().Get("bright");
+
 	// Draw the sidebar.
 	// The currently selected sidebar item should be highlighted. This is how
 	// big the highlight rectangle is.
@@ -105,7 +117,7 @@ void LogbookPanel::Draw()
 	Point highlightOffset = Point(4. - PAD, 0.) + .5 * highlightSize;
 	Point textOffset(0., .5 * (LINE_HEIGHT - font.Height()));
 	// Start at this point on the screen:
-	Point pos = Screen::TopLeft() + Point(PAD, PAD);
+	Point pos = Screen::TopLeft() + Point(PAD, PAD - categoryScroll);
 	for(size_t i = 0; i < contents.size(); ++i)
 	{
 		if(selectedDate ? dates[i].Month() == selectedDate.Month() : selectedName == contents[i])
@@ -116,27 +128,28 @@ void LogbookPanel::Draw()
 		font.Draw(contents[i], pos + textOffset, dates[i].Month() ? medium : bright);
 		pos.Y() += LINE_HEIGHT;
 	}
-	
+
+	maxCategoryScroll = max(0., maxCategoryScroll + pos.Y() - Screen::Bottom());
+
 	// Parameters for drawing the main text:
-	WrappedText wrap;
-	wrap.SetAlignment(WrappedText::JUSTIFIED);
+	WrappedText wrap(font);
+	wrap.SetAlignment(Alignment::JUSTIFIED);
 	wrap.SetWrapWidth(TEXT_WIDTH - 2. * PAD);
-	wrap.SetFont(font);
-	
+
 	// Draw the main text.
 	pos = Screen::TopLeft() + Point(SIDEBAR_WIDTH + PAD, PAD + .5 * (LINE_HEIGHT - font.Height()) - scroll);
-	
+
 	// Branch based on whether this is an ordinary log month or a special page.
 	auto pit = player.SpecialLogs().find(selectedName);
 	if(selectedDate && begin != end)
 	{
+		const auto layout = Layout(static_cast<int>(TEXT_WIDTH - 2. * PAD), Alignment::RIGHT);
 		for(auto it = begin; it != end; ++it)
 		{
 			string date = it->first.ToString();
-			double dateWidth = font.Width(date);
-			font.Draw(date, pos + Point(TEXT_WIDTH - dateWidth - 2. * PAD, textOffset.Y()), dim);
+			font.Draw({date, layout}, pos + Point(0., textOffset.Y()), dim);
 			pos.Y() += LINE_HEIGHT;
-		
+
 			wrap.Wrap(it->second);
 			wrap.Draw(pos, medium);
 			pos.Y() += wrap.Height() + GAP;
@@ -148,19 +161,19 @@ void LogbookPanel::Draw()
 		{
 			font.Draw(it.first, pos + textOffset, bright);
 			pos.Y() += LINE_HEIGHT;
-		
+
 			wrap.Wrap(it.second);
 			wrap.Draw(pos, medium);
 			pos.Y() += wrap.Height() + GAP;
 		}
 	}
-	
+
 	maxScroll = max(0., scroll + pos.Y() - Screen::Bottom());
 }
 
 
 
-bool LogbookPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+bool LogbookPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	if(key == 'd' || key == SDLK_ESCAPE || (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI))))
 		GetUI()->Pop(this);
@@ -169,6 +182,11 @@ bool LogbookPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 		double direction = (key == SDLK_PAGEUP) - (key == SDLK_PAGEDOWN);
 		Drag(0., (Screen::Height() - 100.) * direction);
 	}
+	else if(key == SDLK_HOME || key == SDLK_END)
+	{
+		double direction = (key == SDLK_HOME) - (key == SDLK_END);
+		Drag(0., maxScroll * direction);
+	}
 	else if(key == SDLK_UP || key == SDLK_DOWN)
 	{
 		// Find the index of the currently selected line.
@@ -178,12 +196,12 @@ bool LogbookPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 				break;
 		if(i == contents.size())
 			return true;
-		
+
 		if(key == SDLK_DOWN)
 		{
 			++i;
 			if(i >= contents.size())
-				return true;
+				i = 0;
 		}
 		else if(i)
 		{
@@ -199,15 +217,34 @@ bool LogbookPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 					++i;
 			}
 		}
+		else
+			i = contents.size() - 1;
 		if(contents[i] != selectedName)
 		{
 			selectedDate = dates[i];
 			selectedName = contents[i];
 			scroll = 0.;
 			Update(key == SDLK_UP);
+
+			// Find our currently selected item again
+			for(i = 0 ; i < contents.size(); ++i)
+				if(contents[i] == selectedName)
+					break;
+
+			if(i == contents.size())
+				return true;
+
+			// Check if it's too far down or up
+			int position = i * LINE_HEIGHT - categoryScroll;
+
+			// If it's out of bounds, recenter it
+			if(position < MINIMUM_SELECTION_DISTANCE || position > (Screen::Height() - MINIMUM_SELECTION_DISTANCE))
+				categoryScroll = position - (Screen::Height() / 2);
+
+			categoryScroll = max(categoryScroll, 0.);
 		}
 	}
-	
+
 	return true;
 }
 
@@ -219,7 +256,7 @@ bool LogbookPanel::Click(int x, int y, int clicks)
 	y -= Screen::Top();
 	if(x < SIDEBAR_WIDTH)
 	{
-		size_t index = (y - PAD) / LINE_HEIGHT;
+		size_t index = (y - PAD + categoryScroll) / LINE_HEIGHT;
 		if(index < contents.size())
 		{
 			selectedDate = dates[index];
@@ -232,7 +269,7 @@ bool LogbookPanel::Click(int x, int y, int clicks)
 	}
 	else if(x > WIDTH)
 		GetUI()->Pop(this);
-	
+
 	return true;
 }
 
@@ -240,8 +277,11 @@ bool LogbookPanel::Click(int x, int y, int clicks)
 
 bool LogbookPanel::Drag(double dx, double dy)
 {
-	scroll = max(0., min(maxScroll, scroll - dy));
-	
+	if((hoverPoint.X() - Screen::Left()) > SIDEBAR_WIDTH)
+		scroll = max(0., min(maxScroll, scroll - dy));
+	else
+		categoryScroll = max(0., min(maxCategoryScroll, categoryScroll - dy));
+
 	return true;
 }
 
@@ -254,6 +294,14 @@ bool LogbookPanel::Scroll(double dx, double dy)
 
 
 
+bool LogbookPanel::Hover(int x, int y)
+{
+	hoverPoint = Point(x, y);
+	return true;
+}
+
+
+
 void LogbookPanel::Update(bool selectLast)
 {
 	contents.clear();
@@ -269,7 +317,7 @@ void LogbookPanel::Update(bool selectLast)
 		begin = end = player.Logbook().end();
 		return;
 	}
-	
+
 	// Check what years and months have entries for them.
 	set<int> years;
 	set<int> months;
@@ -279,7 +327,7 @@ void LogbookPanel::Update(bool selectLast)
 		if(it.first.Year() == selectedDate.Year() && it.first.Month() >= 1 && it.first.Month() <= 12)
 			months.insert(it.first.Month());
 	}
-	
+
 	// Generate the table of contents.
 	for(int year : years)
 	{
@@ -298,7 +346,7 @@ void LogbookPanel::Update(bool selectLast)
 		begin = end = player.Logbook().end();
 		return;
 	}
-	
+
 	// Make sure a month is selected, within the current year.
 	if(!selectedDate.Month())
 	{
