@@ -90,10 +90,11 @@ const string &MapOutfitterPanel::KeyLabel(int index) const
 	if(index == 2 && selected && selected->Get("minable") > 0.)
 		return MINE;
 
-	static const string LABEL[3] = {
+	static const string LABEL[4] = {
 		"Has no outfitter",
 		"Has outfitter",
-		"Sells this outfit"
+		"Sells this outfit",
+		"Outfit in storage"
 	};
 	return LABEL[index];
 }
@@ -141,10 +142,14 @@ double MapOutfitterPanel::SystemValue(const System *system) const
 		return numeric_limits<double>::quiet_NaN();
 
 	// Visiting a system is sufficient to know what ports are available on its planets.
-	double value = -.5;
+	double value = -1.;
+	const auto &planetStorage = player.PlanetaryStorage();
 	for(const StellarObject &object : system->Objects())
 		if(object.HasSprite() && object.HasValidPlanet())
 		{
+			const auto storage = planetStorage.find(object.GetPlanet());
+			if(storage != planetStorage.end() && storage->second.Get(selected))
+				return .5;
 			const auto &outfitter = object.GetPlanet()->Outfitter();
 			if(outfitter.Has(selected))
 				return 1.;
@@ -243,11 +248,13 @@ void MapOutfitterPanel::DrawItems()
 					}
 				}
 			}
-			if(!isForSale && !storedInSystem && onlyShowSoldHere)
+			if(!isForSale && onlyShowSoldHere)
+				continue;
+			if(!storedInSystem && onlyShowStorageHere)
 				continue;
 
-			const std::string storage_details =
-				storedInSystem == 0
+			const string storage_details =
+				onlyShowSoldHere || storedInSystem == 0
 				? ""
 				: storedInSystem == 1
 				? "1 unit in storage"
