@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef MENU_PANEL_H_
@@ -18,6 +21,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <string>
 #include <vector>
 
+class Interface;
 class PlayerInfo;
 class UI;
 
@@ -29,23 +33,30 @@ class UI;
 class MenuPanel : public Panel {
 public:
 	MenuPanel(PlayerInfo &player, UI &gamePanels);
-	
+
 	virtual void Step() override;
 	virtual void Draw() override;
-	
-	
+
+
 protected:
 	// Only override the ones you need; the default action is to return false.
 	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress) override;
-	
-	
+	virtual bool Click(int x, int y, int clicks) override;
+
+
+private:
+	void DrawCredits() const;
+
+
 private:
 	PlayerInfo &player;
 	UI &gamePanels;
-	
+
+	const Interface *mainMenuUi;
+
 	std::vector<std::string> credits;
-	unsigned scroll;
-	int progress = 0;
+	long long int scroll = 0;
+	bool scrollingPaused = false;
 };
 
 
