@@ -1,4 +1,4 @@
-/* AI.h
+/* Messages.cpp
 Copyright (c) 2014 by Michael Zahniser
 
 Endless Sky is free software: you can redistribute it and/or modify it under the
@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Messages.h"
@@ -18,18 +21,18 @@ using namespace std;
 
 namespace {
 	mutex incomingMutex;
-	
-	vector<pair<string, bool>> incoming;
+
+	vector<pair<string, Messages::Importance>> incoming;
 	vector<Messages::Entry> list;
 }
 
 
 
-// Add a message to the list.
-void Messages::Add(const string &message, bool isImportant)
+// Add a message to the list along with its level of importance
+void Messages::Add(const string &message, Importance importance)
 {
 	lock_guard<mutex> lock(incomingMutex);
-	incoming.emplace_back(message, isImportant);
+	incoming.emplace_back(message, importance);
 }
 
 
@@ -40,16 +43,16 @@ void Messages::Add(const string &message, bool isImportant)
 const vector<Messages::Entry> &Messages::Get(int step)
 {
 	lock_guard<mutex> lock(incomingMutex);
-	
+
 	// Load the incoming messages.
-	for(const pair<string, bool> &item : incoming)
+	for(const pair<string, Importance> &item : incoming)
 	{
 		const string &message = item.first;
-		bool isImportant = item.second;
-		
+		Importance importance = item.second;
+
 		// If this message is not important and it is already being shown in the
 		// list, ignore it.
-		if(!isImportant)
+		if(importance == Importance::Low)
 		{
 			bool skip = false;
 			for(const Messages::Entry &entry : list)
@@ -57,7 +60,7 @@ const vector<Messages::Entry> &Messages::Get(int step)
 			if(skip)
 				continue;
 		}
-		
+
 		// For each incoming message, if it exactly matches an existing message,
 		// replace that one with this new one.
 		auto it = list.begin();
@@ -67,12 +70,12 @@ const vector<Messages::Entry> &Messages::Get(int step)
 			// limit how many of them appear at once.
 			it->step -= 60;
 			// Also erase messages that have reached the end of their lifetime.
-			if((isImportant && it->message == message) || it->step < step - 1000)
+			if((importance != Importance::Low && it->message == message) || it->step < step - 1000)
 				it = list.erase(it);
 			else
 				++it;
 		}
-		list.emplace_back(step, message);
+		list.emplace_back(step, message, importance);
 	}
 	incoming.clear();
 	return list;
