@@ -7,12 +7,16 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef MORTGAGE_H_
 #define MORTGAGE_H_
 
+#include <cstdint>
 #include <string>
 
 class DataNode;
@@ -29,9 +33,9 @@ class Mortgage {
 public:
 	// Find out how much you can afford to borrow with the given annual revenue
 	// and the given credit score (which should be between 200 and 800).
-	static int64_t Maximum(int64_t annualRevenue, int creditScore, int64_t currentPayments);
-	
-	
+	static int64_t Maximum(int64_t annualRevenue, int creditScore, double currentPayments);
+
+
 public:
 	Mortgage() = default;
 	// Create a new mortgage of the given amount. If this is a fine, set the
@@ -39,11 +43,11 @@ public:
 	Mortgage(int64_t principal, int creditScore, int term = 365);
 	// Construct and Load() at the same time.
 	Mortgage(const DataNode &node);
-	
+
 	// Load or save mortgage data.
 	void Load(const DataNode &node);
 	void Save(DataWriter &out) const;
-	
+
 	// Make a mortgage payment. The return value is the amount paid.
 	int64_t MakePayment();
 	void MissPayment();
@@ -52,7 +56,7 @@ public:
 	// This returns the actual amount paid, which may be less if the total
 	// principal remaining is less than the given amount.
 	int64_t PayExtra(int64_t amount);
-	
+
 	// The type is "Mortgage" if this is a mortgage you applied for from a bank,
 	// and "Fine" if this is a fine imposed on you for illegal activities.
 	const std::string &Type() const;
@@ -65,8 +69,10 @@ public:
 	int Term() const;
 	// Check the amount of the next payment due (rounded to the nearest credit).
 	int64_t Payment() const;
-	
-	
+	// Check the amount of the next payment due.
+	double PrecisePayment() const;
+
+
 private:
 	// Note: once a mortgage is set up, only the principal and term will change.
 	std::string type;
