@@ -34,6 +34,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 class DataNode;
 class DataWriter;
 class Government;
+class Mission;
 class Planet;
 class PlayerInfo;
 class Ship;
@@ -48,6 +49,22 @@ class UI;
 // is not boarded or killed. NPCs also have different "behaviors," such as
 // staying in the system they started in, or attacking only the player's ships.
 class NPC {
+public:
+	enum class Trigger : int_fast8_t {
+		// Triggers corresponding directly to ShipEvents.
+		ASSIST,
+		SCAN_CARGO,
+		SCAN_OUTFITS,
+		PROVOKE,
+		DISABLE,
+		BOARD,
+		CAPTURE,
+		DESTROY,
+		// Can be triggered by either the CAPTURE or DESTROY events.
+		KILL
+	};
+
+
 public:
 	NPC() = default;
 	// Copying an NPC instance isn't allowed.
@@ -58,9 +75,9 @@ public:
 	~NPC() noexcept = default;
 
 	// Construct and Load() at the same time.
-	NPC(const DataNode &node, const std::string &missionName);
+	NPC(const DataNode &node);
 
-	void Load(const DataNode &node, const std::string &missionName);
+	void Load(const DataNode &node);
 	// Note: the Save() function can assume this is an instantiated mission, not
 	// a template, so fleets will be replaced by individual ships already.
 	void Save(DataWriter &out) const;
@@ -79,8 +96,8 @@ public:
 	const std::list<std::shared_ptr<Ship>> Ships() const;
 
 	// Handle the given ShipEvent.
-	enum Trigger {KILL, BOARD, ASSIST, DISABLE, SCAN_CARGO, SCAN_OUTFITS, CAPTURE, PROVOKE};
-	void Do(const ShipEvent &event, PlayerInfo &player, UI *ui = nullptr, bool isVisible = true);
+	void Do(const ShipEvent &event, PlayerInfo &player, UI *ui = nullptr,
+		const Mission *caller = nullptr, bool isVisible = true);
 	// Determine if the NPC is in a successful state, assuming the player is in the given system.
 	// (By default, a despawnable NPC has succeeded and is not actually checked.)
 	bool HasSucceeded(const System *playerSystem, bool ignoreIfDespawnable = true) const;
@@ -98,7 +115,7 @@ public:
 
 private:
 	// Handle any NPC mission actions that may have been triggered by a ShipEvent.
-	void DoActions(const ShipEvent &event, bool newEvent, PlayerInfo &player, UI *ui = nullptr);
+	void DoActions(const ShipEvent &event, bool newEvent, PlayerInfo &player, UI *ui, const Mission *caller);
 
 
 private:
