@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef OUTFIT_INFO_DISPLAY_H_
@@ -15,8 +18,8 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "ItemInfoDisplay.h"
 
-#include <vector>
 #include <string>
+#include <vector>
 
 class PlayerInfo;
 class Point;
@@ -30,29 +33,31 @@ class Outfit;
 class OutfitInfoDisplay : public ItemInfoDisplay {
 public:
 	OutfitInfoDisplay() = default;
-	OutfitInfoDisplay(const Outfit &outfit, const PlayerInfo &player, bool canSell = false);
-	
+	OutfitInfoDisplay(const Outfit &outfit, const PlayerInfo &player,
+			bool canSell = false, bool descriptionCollapsed = true);
+
 	// Call this every time the ship changes.
-	void Update(const Outfit &outfit, const PlayerInfo &player, bool canSell = false);
-	
+	void Update(const Outfit &outfit, const PlayerInfo &player, bool canSell = false, bool descriptionCollapsed = true);
+
 	// Provided by ItemInfoDisplay:
 	// int PanelWidth();
 	// int MaximumHeight() const;
 	// int DescriptionHeight() const;
 	// int AttributesHeight() const;
 	int RequirementsHeight() const;
-	
+
 	// Provided by ItemInfoDisplay:
 	// void DrawDescription(const Point &topLeft) const;
 	// void DrawAttributes(const Point &topLeft) const;
 	void DrawRequirements(const Point &topLeft) const;
-	
-	
+
+
 private:
-	void UpdateRequirements(const Outfit &outfit, const PlayerInfo &player, bool canSell);
+	void UpdateRequirements(const Outfit &outfit, const PlayerInfo &player, bool canSell, bool descriptionCollapsed);
+	void AddRequirementAttribute(std::string label, double value);
 	void UpdateAttributes(const Outfit &outfit);
-	
-	
+
+
 private:
 	std::vector<std::string> requirementLabels;
 	std::vector<std::string> requirementValues;
