@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef OUTLINE_SHADER_H_
@@ -25,8 +28,9 @@ class Sprite;
 class OutlineShader {
 public:
 	static void Init();
-	
-	static void Draw(const Sprite *sprite, const Point &pos, const Point &size, const Color &color, const Point &unit = Point(0., -1.), int frame = 0);
+
+	static void Draw(const Sprite *sprite, const Point &pos, const Point &size,
+		const Color &color, const Point &unit = Point(0., -1.), float frame = 0.f);
 };
 
 
