@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef PERSON_H_
@@ -17,11 +20,13 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Personality.h"
 #include "Phrase.h"
 
+#include <list>
 #include <memory>
 
 class DataNode;
 class Government;
 class Ship;
+class System;
 
 
 
@@ -29,26 +34,38 @@ class Ship;
 class Person {
 public:
 	void Load(const DataNode &node);
-	
+	// Finish loading all the ships in this person specification.
+	void FinishLoading();
+	// Prevent this person from being spawned in any system.
+	void NeverSpawn();
+
 	// Find out how often this person should appear in the given system. If this
 	// person is dead or already active, this will return zero.
 	int Frequency(const System *system) const;
-	
+
 	// Get the person's characteristics. The ship object is persistent, i.e. it
 	// will be recycled every time this person appears.
-	const std::shared_ptr<Ship> &GetShip() const;
+	const std::list<std::shared_ptr<Ship>> &Ships() const;
 	const Government *GetGovernment() const;
 	const Personality &GetPersonality() const;
 	const Phrase &GetHail() const;
 	// Check if a person has been destroyed or captured.
 	bool IsDestroyed() const;
-	
-	
+	// Mark this person as destroyed.
+	void Destroy();
+	// Mark this person as no longer destroyed.
+	void Restore();
+	// Check if a person is already placed somewhere.
+	bool IsPlaced() const;
+	// Mark this person as being no longer "placed" somewhere.
+	void ClearPlacement();
+
+
 private:
 	LocationFilter location;
 	int frequency = 100;
-	
-	std::shared_ptr<Ship> ship;
+
+	std::list<std::shared_ptr<Ship>> ships;
 	const Government *government = nullptr;
 	Personality personality;
 	Phrase hail;
