@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Phrase.h"
@@ -15,12 +18,48 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "DataNode.h"
 #include "text/Format.h"
 #include "GameData.h"
-#include "Random.h"
 
 using namespace std;
 
 
 
+// Replace all occurrences ${phrase name} with the expanded phrase from GameData::Phrases()
+std::string Phrase::ExpandPhrases(const std::string &source)
+{
+	string result;
+	size_t next = 0;
+	while(next < source.length())
+	{
+		size_t var = source.find("${", next);
+		if(var == string::npos)
+			break;
+		else if(var > next)
+			result.append(source, next, var - next);
+		next = source.find('}', var);
+		if(next == string::npos)
+			break;
+		++next;
+		string phraseName = string{source, var + 2, next - var - 3};
+		const Phrase *phrase = GameData::Phrases().Find(phraseName);
+		result.append(phrase ? phrase->Get() : phraseName);
+	}
+	// Optimization for most common case: no phrase in string:
+	if(!next)
+		return source;
+	else if(next < source.length())
+		result.append(source, next, string::npos);
+	return result;
+}
+
+
+
+Phrase::Phrase(const DataNode &node)
+{
+	Load(node);
+}
+
+
+
 void Phrase::Load(const DataNode &node)
 {
 	// Set the name of this phrase, so we know it has been loaded.
@@ -29,15 +68,15 @@ void Phrase::Load(const DataNode &node)
 	// may not be used in a Phrase's name.
 	if(name.find("${") != string::npos || name.find('}') != string::npos)
 	{
-		node.PrintTrace("Phrase names may not contain '${' or '}':");
+		node.PrintTrace("Error: Phrase names may not contain '${' or '}':");
 		return;
 	}
-	
+
 	sentences.emplace_back(node, this);
 	if(sentences.back().empty())
 	{
 		sentences.pop_back();
-		node.PrintTrace("Skipping unparseable node:");
+		node.PrintTrace("Error: Unable to parse node:");
 	}
 }
 
@@ -65,12 +104,12 @@ string Phrase::Get() const
 	string result;
 	if(sentences.empty())
 		return result;
-	
+
 	for(const auto &part : sentences[Random::Int(sentences.size())])
 	{
 		if(!part.choices.empty())
 		{
-			const auto &choice = part.choices[Random::Int(part.choices.size())];
+			const auto &choice = part.choices.Get();
 			for(const auto &element : choice)
 				result += element.second ? element.second->Get() : element.first;
 		}
@@ -78,7 +117,7 @@ string Phrase::Get() const
 			for(const auto &pair : part.replacements)
 				Format::ReplaceAll(result, pair.first, pair.second);
 	}
-	
+
 	return result;
 }
 
@@ -90,14 +129,14 @@ bool Phrase::ReferencesPhrase(const Phrase *other) const
 {
 	if(other == this)
 		return true;
-	
+
 	for(const auto &sentence : sentences)
 		for(const auto &part : sentence)
 			for(const auto &choice : part.choices)
 				for(const auto &element : choice)
 					if(element.second && element.second->ReferencesPhrase(other))
 						return true;
-	
+
 	return false;
 }
 
@@ -114,7 +153,7 @@ Phrase::Choice::Choice(const DataNode &node, bool isPhraseName)
 		emplace_back(string{}, GameData::Phrases().Get(node.Token(0)));
 		return;
 	}
-	
+
 	// This node is a text string that may contain an interpolation request.
 	const string &entry = node.Token(0);
 	if(entry.empty())
@@ -123,7 +162,7 @@ Phrase::Choice::Choice(const DataNode &node, bool isPhraseName)
 		emplace_back();
 		return;
 	}
-	
+
 	size_t start = 0;
 	while(start < entry.length())
 	{
@@ -134,7 +173,7 @@ Phrase::Choice::Choice(const DataNode &node, bool isPhraseName)
 		size_t right = entry.find('}', left);
 		if(right == string::npos)
 			break;
-		
+
 		// Add the text up to the ${, and then add the contained phrase name.
 		++right;
 		size_t length = right - left;
@@ -169,32 +208,32 @@ void Phrase::Sentence::Load(const DataNode &node, const Phrase *parent)
 			child.PrintTrace("Skipping node with no children:");
 			continue;
 		}
-		
+
 		emplace_back();
 		auto &part = back();
-		
+
 		if(child.Token(0) == "word")
 			for(const DataNode &grand : child)
-				part.choices.emplace_back(grand);
+				part.choices.emplace_back((grand.Size() >= 2) ? max<int>(1, grand.Value(1)) : 1, grand);
 		else if(child.Token(0) == "phrase")
 			for(const DataNode &grand : child)
-				part.choices.emplace_back(grand, true);
+				part.choices.emplace_back((grand.Size() >= 2) ? max<int>(1, grand.Value(1)) : 1, grand, true);
 		else if(child.Token(0) == "replace")
 			for(const DataNode &grand : child)
-				part.replacements.emplace_back(grand.Token(0), grand.Size() >= 2 ? grand.Token(1) : string{});
+				part.replacements.emplace_back(grand.Token(0), (grand.Size() >= 2) ? grand.Token(1) : string{});
 		else
 			child.PrintTrace("Skipping unrecognized attribute:");
-		
+
 		// Require any newly added phrases have no recursive references. Any recursions
 		// will instead yield an empty string, rather than possibly infinite text.
 		for(auto &choice : part.choices)
 			for(auto &element : choice)
 				if(element.second && element.second->ReferencesPhrase(parent))
 				{
-					child.PrintTrace("Replaced recursive '" + element.second->Name() + "' phrase reference with \"\":");
+					child.PrintTrace("Warning: Replaced recursive '" + element.second->Name() + "' phrase reference with \"\":");
 					element.second = nullptr;
 				}
-		
+
 		// If no words, phrases, or replaces were given, discard this part of the phrase.
 		if(part.choices.empty() && part.replacements.empty())
 			pop_back();
