@@ -79,10 +79,12 @@ void PlanetPanel::Step()
 		return;
 	}
 
-	// If the player starts a new game, exits the shipyard without buying
-	// anything, clicks to the bank, then returns to the shipyard and buys a
-	// ship, make sure they are shown an intro mission.
-	if(GetUI()->IsTop(this) || GetUI()->IsTop(bank.get()))
+	// Handle missions for locations that aren't handled separately,
+	// treating them all as the landing location. This is mainly to
+	// handle the intro mission in the event the player moves away
+	// from the landing before buying a ship.
+	const Panel *activePanel = selectedPanel ? selectedPanel : this;
+	if(activePanel != spaceport.get() && GetUI()->IsTop(activePanel))
 	{
 		Mission *mission = player.MissionToOffer(Mission::LANDING);
 		if(mission)
@@ -110,28 +112,20 @@ void PlanetPanel::Draw()
 	{
 		if(planet.IsInhabited())
 		{
+			info.SetCondition("is inhabited");
 			info.SetCondition("has bank");
-			if(flagship)
-			{
-				info.SetCondition("is inhabited");
-				if(system.HasTrade())
-					info.SetCondition("has trade");
-			}
+			if(system.HasTrade())
+				info.SetCondition("has trade");
 		}
 
-		if(flagship && planet.HasSpaceport())
+		if(planet.HasSpaceport())
 			info.SetCondition("has spaceport");
 
 		if(planet.HasShipyard())
 			info.SetCondition("has shipyard");
 
 		if(planet.HasOutfitter())
-			for(const auto &it : player.Ships())
-				if(it->GetSystem() == &system && !it->IsDisabled())
-				{
-					info.SetCondition("has outfitter");
-					break;
-				}
+			info.SetCondition("has outfitter");
 	}
 
 	ui.Draw(info, this);
@@ -164,7 +158,7 @@ bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, b
 	}
 	else if(key == 'l')
 		selectedPanel = nullptr;
-	else if(key == 't' && hasAccess && flagship && planet.IsInhabited() && system.HasTrade())
+	else if(key == 't' && hasAccess && planet.IsInhabited() && system.HasTrade())
 	{
 		selectedPanel = trading.get();
 		GetUI()->Push(trading);
@@ -174,7 +168,7 @@ bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, b
 		selectedPanel = bank.get();
 		GetUI()->Push(bank);
 	}
-	else if(key == 'p' && hasAccess && flagship && planet.HasSpaceport())
+	else if(key == 'p' && hasAccess && planet.HasSpaceport())
 	{
 		selectedPanel = spaceport.get();
 		if(isNewPress)
@@ -188,19 +182,15 @@ bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, b
 	}
 	else if(key == 'o' && hasAccess && planet.HasOutfitter())
 	{
-		for(const auto &it : player.Ships())
-			if(it->GetSystem() == &system && !it->IsDisabled())
-			{
-				GetUI()->Push(new OutfitterPanel(player));
-				return true;
-			}
+		GetUI()->Push(new OutfitterPanel(player));
+		return true;
 	}
-	else if(key == 'j' && hasAccess && flagship && planet.IsInhabited())
+	else if(key == 'j' && hasAccess && planet.IsInhabited())
 	{
 		GetUI()->Push(new MissionPanel(player));
 		return true;
 	}
-	else if(key == 'h' && hasAccess && flagship && planet.IsInhabited())
+	else if(key == 'h' && hasAccess && planet.IsInhabited())
 	{
 		selectedPanel = hiring.get();
 		GetUI()->Push(hiring);
@@ -252,7 +242,7 @@ void PlanetPanel::TakeOffIfReady()
 	// Check whether the player can be warned before takeoff.
 	if(player.ShouldLaunch())
 	{
-		TakeOff();
+		TakeOff(true);
 		return;
 	}
 
@@ -280,7 +270,7 @@ void PlanetPanel::TakeOffIfReady()
 				else
 				{
 					GetUI()->Push(new ConversationPanel(player,
-						*GameData::Conversations().Get("flight check: " + check), nullptr, result.first));
+						*GameData::Conversations().Get("flight check: " + check), nullptr, nullptr, result.first));
 					return;
 				}
 			}
@@ -312,13 +302,26 @@ void PlanetPanel::CheckWarningsAndTakeOff()
 
 	// Check for items that would be sold, or mission passengers that would be abandoned on-planet.
 	const Ship *flagship = player.Flagship();
-	const CargoHold &cargo = player.Cargo();
+	const CargoHold &cargo = player.DistributeCargo();
 	// Are you overbooked? Don't count fireable flagship crew.
 	// (If your ship can't support its required crew, it is counted as having no fireable crew.)
-	int overbooked = -cargo.BunksFree() - max(0, flagship->Crew() - flagship->RequiredCrew());
-	int missionCargoToSell = cargo.MissionCargoSize() - cargo.Size();
+	const int overbooked = cargo.Passengers() - max(0, flagship->Crew() - flagship->RequiredCrew());
+	const int missionCargoToSell = cargo.MissionCargoSize();
 	// Will you have to sell something other than regular cargo?
-	int cargoToSell = -(cargo.Free() + cargo.CommoditiesSize());
+	const int commoditiesToSell = cargo.CommoditiesSize();
+	int outfitsToSell = 0;
+	map<const Outfit *, int> uniquesToSell;
+	for(auto &it : cargo.Outfits())
+	{
+		outfitsToSell += it.second;
+		if(it.first->Attributes().Get("unique"))
+			uniquesToSell[it.first] = it.second;
+	}
+	// Have you left any unique items at the outfitter?
+	map<const Outfit *, int> leftUniques;
+	for(const auto &it : player.GetStock())
+		if(it.second > 0 && it.first->Attributes().Get("unique"))
+			leftUniques[it.first] = it.second;
 	// Count how many active ships we have that cannot make the jump (e.g. due to lack of fuel,
 	// drive, or carrier). All such ships will have been logged in the player's flightcheck.
 	size_t nonJumpCount = 0;
@@ -337,59 +340,109 @@ void PlanetPanel::CheckWarningsAndTakeOff()
 				}
 	}
 
-	if(nonJumpCount > 0 || cargoToSell > 0 || overbooked > 0)
+	if(nonJumpCount > 0 || missionCargoToSell > 0 || outfitsToSell > 0 || commoditiesToSell > 0 || overbooked > 0
+		|| !leftUniques.empty())
 	{
 		ostringstream out;
+		auto ListUniques = [&out] (const map<const Outfit *, int> uniques)
+		{
+			const int detailedSize = (uniques.size() > 5 ? 4 : uniques.size());
+			auto it = uniques.begin();
+			for(int i = 0; i < detailedSize; ++i)
+			{
+				out << "\n" + to_string(it->second) + " "
+					+ (it->second == 1 ? it->first->DisplayName() : it->first->PluralName());
+				++it;
+			}
+			int otherUniquesCount = 0;
+			if(it != uniques.end())
+			{
+				for( ; it != uniques.end(); ++it)
+					otherUniquesCount += it->second;
+				out << "\nand " + to_string(otherUniquesCount) + " other unique outfits.";
+			}
+			else
+				out << ".";
+		};
+		out << "If you take off now, you will:";
+
 		// Warn about missions that will fail on takeoff.
 		if(missionCargoToSell > 0 || overbooked > 0)
 		{
-			bool both = ((cargoToSell > 0 && cargo.MissionCargoSize()) && overbooked > 0);
-			out << "If you take off now you will abort a mission due to not having enough ";
+			out << "\n- abort a mission due to not having enough ";
 
 			if(overbooked > 0)
 			{
 				out << "bunks available for " << overbooked;
 				out << (overbooked > 1 ? " of the passengers" : " passenger");
-				out << (both ? " and not having enough " : ".");
+				out << (missionCargoToSell > 0 ? " and not having enough " : ".");
 			}
 
 			if(missionCargoToSell > 0)
-				out << "cargo space to hold " << Format::CargoString(missionCargoToSell, "your mission cargo") << ".";
+				out << "cargo space to hold " << Format::CargoString(missionCargoToSell, "mission cargo.");
+		}
+		// Warn about outfits that can't be carried.
+		if(outfitsToSell > 0)
+		{
+			out << "\n- ";
+			out << (planet.HasOutfitter() ? "store " : "sell ") << outfitsToSell << " outfit";
+			out << (outfitsToSell > 1 ? "s" : "");
+			out << " that none of your ships can hold.";
+			if(!uniquesToSell.empty())
+			{
+				out << " Some of the outfits are unique:";
+				ListUniques(uniquesToSell);
+			}
+		}
+		// Warn about unique items you sold.
+		if(!leftUniques.empty())
+		{
+			out << "\n- not be able to re-purchase unique outfits you sold at the outfitter:";
+			ListUniques(leftUniques);
 		}
 		// Warn about ships that won't travel with you.
-		else if(nonJumpCount > 0)
+		if(nonJumpCount > 0)
 		{
-			out << "If you take off now you will launch with ";
+			out << "\n- launch with ";
 			if(nonJumpCount == 1)
 				out << "a ship";
 			else
 				out << nonJumpCount << " ships";
 			out << " that will not be able to leave the system.";
 		}
-		// Warn about non-commodity cargo you will have to sell.
-		else
+		// Warn about commodities you will have to sell.
+		if(commoditiesToSell > 0)
 		{
-			out << "If you take off now you will have to sell ";
-			out << Format::CargoString(cargoToSell, "cargo");
+			out << "\n- sell " << Format::CargoString(commoditiesToSell, "cargo");
 			out << " that you do not have space for.";
 		}
-		out << " Are you sure you want to continue?";
-		GetUI()->Push(new Dialog(this, &PlanetPanel::TakeOff, out.str()));
+		out << "\nAre you sure you want to continue?";
+		GetUI()->Push(new Dialog(this, &PlanetPanel::WarningsDialogCallback, out.str()));
 		return;
 	}
 
 	// There was no need to ask the player whether we can get rid of anything,
 	// so go ahead and take off.
-	TakeOff();
+	TakeOff(false);
 }
 
 
 
-void PlanetPanel::TakeOff()
+void PlanetPanel::WarningsDialogCallback(const bool isOk)
+{
+	if(isOk)
+		TakeOff(false);
+	else
+		player.PoolCargo();
+}
+
+
+
+void PlanetPanel::TakeOff(const bool distributeCargo)
 {
 	flightChecks.clear();
 	player.Save();
-	if(player.TakeOff(GetUI()))
+	if(player.TakeOff(GetUI(), distributeCargo))
 	{
 		if(callback)
 			callback();
