@@ -77,6 +77,9 @@ void Politics::Reset()
 
 bool Politics::IsEnemy(const Government *first, const Government *second) const
 {
+	if(!first || !second)
+		return false;
+
 	if(first == second)
 		return false;
 
@@ -108,6 +111,9 @@ bool Politics::IsEnemy(const Government *first, const Government *second) const
 // reputation.
 void Politics::Offend(const Government *gov, int eventType, int count)
 {
+	if(!gov)
+		return;
+
 	if(gov->IsPlayer())
 		return;
 
@@ -160,12 +166,11 @@ bool Politics::CanLand(const Ship &ship, const Planet *planet) const
 {
 	if(!planet || !planet->GetSystem())
 		return false;
-	if(!planet->IsInhabited())
-		return true;
 
 	const Government *gov = ship.GetGovernment();
 	if(!gov->IsPlayer())
-		return !IsEnemy(gov, planet->GetGovernment());
+		return (ship.GetPersonality().IsUnrestricted() || !gov->IsRestrictedFrom(*planet)) &&
+			(!planet->IsInhabited() || !IsEnemy(gov, planet->GetGovernment()));
 
 	return CanLand(planet);
 }
@@ -280,6 +285,7 @@ string Politics::Fine(PlayerInfo &player, const Government *gov, int scan, const
 			}
 		}
 		if((!scan || (scan & ShipEvent::SCAN_OUTFITS)) && !EvadesOutfitScan(*ship))
+		{
 			for(const auto &it : ship->Outfits())
 				if(it.second)
 				{
@@ -292,9 +298,20 @@ string Politics::Fine(PlayerInfo &player, const Government *gov, int scan, const
 						reason = " for having illegal outfits installed on your ship.";
 					}
 				}
+
+			int shipFine = gov->Fines(ship.get());
+			if(gov->Condemns(ship.get()))
+				shipFine = -1;
+			if((shipFine > maxFine && maxFine >= 0) || shipFine < 0)
+			{
+				maxFine = shipFine;
+				reason = " for flying an illegal ship.";
+			}
+		}
 		if(failedMissions && maxFine > 0)
 		{
-			reason += "\n\tYou failed " + Format::Number(failedMissions) + ((failedMissions > 1) ? " missions" : " mission")
+			reason += "\n\tYou failed " + Format::Number(failedMissions)
+				+ ((failedMissions > 1) ? " missions" : " mission")
 				+ " after your illegal cargo was discovered.";
 		}
 	}
