@@ -20,6 +20,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "Color.h"
 #include "Dialog.h"
 #include "Files.h"
+#include "FillShader.h"
 #include "text/Font.h"
 #include "text/FontSet.h"
 #include "GameData.h"
@@ -62,6 +63,7 @@ namespace {
 	const string STATUS_OVERLAYS_ENEMY = "   Show enemy overlays";
 	const string STATUS_OVERLAYS_NEUTRAL = "   Show neutral overlays";
 	const string EXPEND_AMMO = "Escorts expend ammo";
+	const string FLOTSAM_SETTING = "Flotsam collection";
 	const string TURRET_TRACKING = "Turret tracking";
 	const string FOCUS_PREFERENCE = "Turrets focus fire";
 	const string FRUGAL_ESCORTS = "Escorts use ammo frugally";
@@ -69,13 +71,17 @@ namespace {
 	const string SCROLL_SPEED = "Scroll speed";
 	const string FIGHTER_REPAIR = "Repair fighters in";
 	const string SHIP_OUTLINES = "Ship outlines in shops";
+	const string DATE_FORMAT = "Date format";
 	const string BOARDING_PRIORITY = "Boarding target priority";
 	const string TARGET_ASTEROIDS_BASED_ON = "Target asteroid based on";
 	const string BACKGROUND_PARALLAX = "Parallax background";
+	const string EXTENDED_JUMP_EFFECTS = "Extended jump effects";
 	const string ALERT_INDICATOR = "Alert indicator";
 
 	// How many pages of settings there are.
 	const int SETTINGS_PAGE_COUNT = 2;
+	// Hovering a preference for this many frames activates the tooltip.
+	const int HOVER_TIME = 60;
 }
 
 
@@ -92,6 +98,11 @@ PreferencesPanel::PreferencesPanel()
 		}
 
 	SetIsFullScreen(true);
+
+	// Initialize a centered tooltip.
+	hoverText.SetFont(FontSet::Get(14));
+	hoverText.SetWrapWidth(250);
+	hoverText.SetAlignment(Alignment::LEFT);
 }
 
 
@@ -121,9 +132,15 @@ void PreferencesPanel::Draw()
 	prefZones.clear();
 	pluginZones.clear();
 	if(page == 'c')
+	{
 		DrawControls();
+		DrawTooltips();
+	}
 	else if(page == 's')
+	{
 		DrawSettings();
+		DrawTooltips();
+	}
 	else if(page == 'p')
 		DrawPlugins();
 }
@@ -148,7 +165,10 @@ bool PreferencesPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &comma
 	else if(key == 'b' || command.Has(Command::MENU) || (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI))))
 		Exit();
 	else if(key == 'c' || key == 's' || key == 'p')
+	{
 		page = key;
+		hoverItem.clear();
+	}
 	else if(key == 'o' && page == 'p')
 		Files::OpenUserPluginFolder();
 	else if((key == 'n' || key == SDLK_PAGEUP) && currentSettingsPage < SETTINGS_PAGE_COUNT - 1)
@@ -212,6 +232,8 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 				Preferences::ToggleBoarding();
 			else if(zone.Value() == BACKGROUND_PARALLAX)
 				Preferences::ToggleParallax();
+			else if(zone.Value() == EXTENDED_JUMP_EFFECTS)
+				Preferences::ToggleExtendedJumpEffects();
 			else if(zone.Value() == VIEW_ZOOM_FACTOR)
 			{
 				// Increase the zoom factor unless it is at the maximum. In that
@@ -243,6 +265,8 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 				Preferences::ToggleAutoFire();
 			else if(zone.Value() == EXPEND_AMMO)
 				Preferences::ToggleAmmoUsage();
+			else if(zone.Value() == FLOTSAM_SETTING)
+				Preferences::ToggleFlotsam();
 			else if(zone.Value() == TURRET_TRACKING)
 				Preferences::Set(FOCUS_PREFERENCE, !Preferences::Has(FOCUS_PREFERENCE));
 			else if(zone.Value() == REACTIVATE_HELP)
@@ -258,6 +282,8 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 					speed = 20;
 				Preferences::SetScrollSpeed(speed);
 			}
+			else if(zone.Value() == DATE_FORMAT)
+				Preferences::ToggleDateFormat();
 			else if(zone.Value() == ALERT_INDICATOR)
 				Preferences::ToggleAlert();
 			// All other options are handled by just toggling the boolean state.
@@ -282,20 +308,21 @@ bool PreferencesPanel::Hover(int x, int y)
 {
 	hoverPoint = Point(x, y);
 
+	hoverItem.clear();
+	tooltip.clear();
+
 	hover = -1;
 	for(unsigned index = 0; index < zones.size(); ++index)
 		if(zones[index].Contains(hoverPoint))
 			hover = index;
 
-	hoverPreference.clear();
 	for(const auto &zone : prefZones)
 		if(zone.Contains(hoverPoint))
-			hoverPreference = zone.Value();
+			hoverItem = zone.Value();
 
-	hoverPlugin.clear();
 	for(const auto &zone : pluginZones)
 		if(zone.Contains(hoverPoint))
-			hoverPlugin = zone.Value();
+			hoverItem = zone.Value();
 
 	return true;
 }
@@ -305,10 +332,10 @@ bool PreferencesPanel::Hover(int x, int y)
 // Change the value being hovered over in the direction of the scroll.
 bool PreferencesPanel::Scroll(double dx, double dy)
 {
-	if(!dy || hoverPreference.empty())
+	if(!dy || page != 's' || hoverItem.empty())
 		return false;
 
-	if(hoverPreference == ZOOM_FACTOR)
+	if(hoverItem == ZOOM_FACTOR)
 	{
 		int zoom = Screen::UserZoom();
 		if(dy < 0. && zoom > ZOOM_FACTOR_MIN)
@@ -325,14 +352,14 @@ bool PreferencesPanel::Scroll(double dx, double dy)
 		point += .5 * Point(Screen::RawWidth(), Screen::RawHeight());
 		SDL_WarpMouseInWindow(nullptr, point.X(), point.Y());
 	}
-	else if(hoverPreference == VIEW_ZOOM_FACTOR)
+	else if(hoverItem == VIEW_ZOOM_FACTOR)
 	{
 		if(dy < 0.)
 			Preferences::ZoomViewOut();
 		else
 			Preferences::ZoomViewIn();
 	}
-	else if(hoverPreference == SCROLL_SPEED)
+	else if(hoverItem == SCROLL_SPEED)
 	{
 		int speed = Preferences::ScrollSpeed();
 		if(dy < 0.)
@@ -419,6 +446,7 @@ void PreferencesPanel::DrawControls()
 		Command::MENU,
 		Command::FULLSCREEN,
 		Command::FASTFORWARD,
+		Command::HELP,
 		Command::NONE,
 		Command::DEPLOY,
 		Command::FIGHT,
@@ -469,7 +497,10 @@ void PreferencesPanel::DrawControls()
 			// Highlight whichever row the mouse hovers over.
 			table.SetHighlight(-120, 120);
 			if(isHovering)
+			{
 				table.DrawHighlight(back);
+				hoverItem = command.Description();
+			}
 
 			zones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), command);
 
@@ -478,18 +509,18 @@ void PreferencesPanel::DrawControls()
 		}
 	}
 
-	Table shiftTable;
-	shiftTable.AddColumn(125, {150, Alignment::RIGHT});
-	shiftTable.SetUnderline(0, 130);
-	shiftTable.DrawAt(Point(-400, 32));
+	Table infoTable;
+	infoTable.AddColumn(125, {150, Alignment::RIGHT});
+	infoTable.SetUnderline(0, 130);
+	infoTable.DrawAt(Point(-400, 32));
 
-	shiftTable.DrawUnderline(medium);
-	shiftTable.Draw("With <shift> key", bright);
-	shiftTable.DrawGap(5);
-	shiftTable.Draw("Select nearest ship", medium);
-	shiftTable.Draw("Select next escort", medium);
-	shiftTable.Draw("Talk to planet", medium);
-	shiftTable.Draw("Board disabled escort", medium);
+	infoTable.DrawUnderline(medium);
+	infoTable.Draw("Additional info", bright);
+	infoTable.DrawGap(5);
+	infoTable.Draw("Press '_x' over controls", medium);
+	infoTable.Draw("to unbind them.", medium);
+	infoTable.Draw("Controls can share", medium);
+	infoTable.Draw("the same keybind.", medium);
 }
 
 
@@ -536,6 +567,7 @@ void PreferencesPanel::DrawSettings()
 		"Draw starfield",
 		BACKGROUND_PARALLAX,
 		"Show hyperspace flash",
+		EXTENDED_JUMP_EFFECTS,
 		SHIP_OUTLINES,
 		"\t",
 		"HUD",
@@ -561,13 +593,14 @@ void PreferencesPanel::DrawSettings()
 		TURRET_TRACKING,
 		TARGET_ASTEROIDS_BASED_ON,
 		BOARDING_PRIORITY,
-		"Flagship flotsam collection",
 		EXPEND_AMMO,
+		FLOTSAM_SETTING,
 		FIGHTER_REPAIR,
 		"Fighters transfer cargo",
 		"Rehire extra crew when lost",
 		"",
 		"Map",
+		"Deadline blink by distance",
 		"Hide unexplored map regions",
 		"Show escort systems on map",
 		"Show stored outfits on map",
@@ -577,7 +610,9 @@ void PreferencesPanel::DrawSettings()
 		"Always underline shortcuts",
 		REACTIVATE_HELP,
 		"Interrupt fast-forward",
-		SCROLL_SPEED
+		"Landing zoom",
+		SCROLL_SPEED,
+		DATE_FORMAT
 	};
 
 	bool isCategory = true;
@@ -683,6 +718,16 @@ void PreferencesPanel::DrawSettings()
 		}
 		else if(setting == EXPEND_AMMO)
 			text = Preferences::AmmoUsage();
+		else if(setting == DATE_FORMAT)
+		{
+			text = Preferences::DateFormatSetting();
+			isOn = true;
+		}
+		else if(setting == FLOTSAM_SETTING)
+		{
+			text = Preferences::FlotsamSetting();
+			isOn = text != "off";
+		}
 		else if(setting == TURRET_TRACKING)
 		{
 			isOn = true;
@@ -713,6 +758,11 @@ void PreferencesPanel::DrawSettings()
 			text = Preferences::ParallaxSetting();
 			isOn = text != "off";
 		}
+		else if(setting == EXTENDED_JUMP_EFFECTS)
+		{
+			text = Preferences::ExtendedJumpEffectsSetting();
+			isOn = text != "off";
+		}
 		else if(setting == REACTIVATE_HELP)
 		{
 			// Check how many help messages have been displayed.
@@ -757,7 +807,7 @@ void PreferencesPanel::DrawSettings()
 		else
 			text = isOn ? "on" : "off";
 
-		if(setting == hoverPreference)
+		if(setting == hoverItem)
 			table.DrawHighlight(back);
 		table.Draw(setting, isOn ? medium : dim);
 		table.Draw(text, isOn ? bright : medium);
@@ -775,7 +825,7 @@ void PreferencesPanel::DrawPlugins()
 
 	const Sprite *box[2] = { SpriteSet::Get("ui/unchecked"), SpriteSet::Get("ui/checked") };
 
-	const int MAX_TEXT_WIDTH = 230;
+	const int MAX_TEXT_WIDTH = 210;
 	Table table;
 	table.AddColumn(-115, {MAX_TEXT_WIDTH, Truncate::MIDDLE});
 	table.SetUnderline(-120, 100);
@@ -797,7 +847,7 @@ void PreferencesPanel::DrawPlugins()
 		pluginZones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), plugin.name);
 
 		bool isSelected = (plugin.name == selectedPlugin);
-		if(isSelected || plugin.name == hoverPlugin)
+		if(isSelected || plugin.name == hoverItem)
 			table.DrawHighlight(back);
 
 		const Sprite *sprite = box[plugin.currentState];
@@ -837,6 +887,46 @@ void PreferencesPanel::DrawPlugins()
 
 
 
+void PreferencesPanel::DrawTooltips()
+{
+	if(hoverItem.empty())
+	{
+		// Step the tooltip timer back.
+		hoverCount -= hoverCount ? 1 : 0;
+		return;
+	}
+
+	// Step the tooltip timer forward [0-60].
+	hoverCount += hoverCount < HOVER_TIME;
+
+	if(hoverCount < HOVER_TIME)
+		return;
+
+	// Create the tooltip text.
+	if(tooltip.empty())
+	{
+		tooltip = GameData::Tooltip(hoverItem);
+		// No tooltip for this item.
+		if(tooltip.empty())
+			return;
+		hoverText.Wrap(tooltip);
+	}
+
+	Point size(hoverText.WrapWidth(), hoverText.Height() - hoverText.ParagraphBreak());
+	size += Point(20., 20.);
+	Point topLeft = hoverPoint;
+	// Do not overflow the screen dimensions.
+	if(topLeft.X() + size.X() > Screen::Right())
+		topLeft.X() -= size.X();
+	if(topLeft.Y() + size.Y() > Screen::Bottom())
+		topLeft.Y() -= size.Y();
+	// Draw the background fill and the tooltip text.
+	FillShader::Fill(topLeft + .5 * size, size, *GameData::Colors().Get("tooltip background"));
+	hoverText.Draw(topLeft + Point(10., 10.), *GameData::Colors().Get("medium"));
+}
+
+
+
 void PreferencesPanel::Exit()
 {
 	Command::SaveSettings(Files::Config() + "keys.txt");
