@@ -45,10 +45,11 @@ ShipInfoDisplay::ShipInfoDisplay(const Ship &ship, const PlayerInfo &player, boo
 
 
 // Call this every time the ship changes.
-void ShipInfoDisplay::Update(const Ship &ship, const PlayerInfo &player, bool descriptionCollapsed)
+// Panels that have scrolling abilities are not limited by space, allowing more detailed attributes.
+void ShipInfoDisplay::Update(const Ship &ship, const PlayerInfo &player, bool descriptionCollapsed, bool scrollingPanel)
 {
 	UpdateDescription(ship.Description(), ship.Attributes().Licenses(), true);
-	UpdateAttributes(ship, player, descriptionCollapsed);
+	UpdateAttributes(ship, player, descriptionCollapsed, scrollingPanel);
 	const Depreciation &depreciation = ship.IsYours() ? player.FleetDepreciation() : player.StockDepreciation();
 	UpdateOutfits(ship, player, depreciation);
 
@@ -132,7 +133,8 @@ void ShipInfoDisplay::DrawOutfits(const Point &topLeft) const
 
 
 
-void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &player, bool descriptionCollapsed)
+void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &player, bool descriptionCollapsed,
+		bool scrollingPanel)
 {
 	bool isGeneric = ship.Name().empty() || ship.GetPlanet();
 
@@ -140,7 +142,7 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	attributeHeaderValues.clear();
 
 	attributeHeaderLabels.push_back("model:");
-	attributeHeaderValues.push_back(ship.ModelName());
+	attributeHeaderValues.push_back(ship.DisplayModelName());
 	attributesHeight = 20;
 
 	attributeLabels.clear();
@@ -187,13 +189,13 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	if(hasShieldRegen)
 	{
 		attributeLabels.push_back("shields (charge):");
-		attributeValues.push_back(Format::Number(attributes.Get("shields"))
+		attributeValues.push_back(Format::Number(ship.MaxShields())
 			+ " (" + Format::Number(60. * shieldRegen) + "/s)");
 	}
 	else
 	{
 		attributeLabels.push_back("shields:");
-		attributeValues.push_back(Format::Number(attributes.Get("shields")));
+		attributeValues.push_back(Format::Number(ship.MaxShields()));
 	}
 	attributesHeight += 20;
 	double hullRepair = attributes.Get("hull repair rate")
@@ -202,13 +204,13 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	if(hasHullRepair)
 	{
 		attributeLabels.push_back("hull (repair):");
-		attributeValues.push_back(Format::Number(attributes.Get("hull"))
+		attributeValues.push_back(Format::Number(ship.MaxHull())
 			+ " (" + Format::Number(60. * hullRepair) + "/s)");
 	}
 	else
 	{
 		attributeLabels.push_back("hull:");
-		attributeValues.push_back(Format::Number(attributes.Get("hull")));
+		attributeValues.push_back(Format::Number(ship.MaxHull()));
 	}
 	attributesHeight += 20;
 	double emptyMass = attributes.Mass();
@@ -249,7 +251,7 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	attributeValues.push_back(Format::Number(60. * forwardThrust / ship.Drag()));
 	attributesHeight += 20;
 
-	// Movement stats are influenced by inertia redeuction.
+	// Movement stats are influenced by inertia reduction.
 	double reduction = 1. + attributes.Get("inertia reduction");
 	emptyMass /= reduction;
 	currentMass /= reduction;
@@ -332,6 +334,7 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	energyTable.push_back(Format::Number(60. * idleEnergyPerFrame));
 	heatTable.push_back(Format::Number(60. * idleHeatPerFrame));
 
+	// Add energy and heat while moving to the table.
 	attributesHeight += 20;
 	const double movingEnergyPerFrame =
 		max(attributes.Get("thrusting energy"), attributes.Get("reverse thrusting energy"))
@@ -344,6 +347,7 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	energyTable.push_back(Format::Number(-60. * movingEnergyPerFrame));
 	heatTable.push_back(Format::Number(60. * movingHeatPerFrame));
 
+	// Add energy and heat while firing to the table.
 	attributesHeight += 20;
 	double firingEnergy = 0.;
 	double firingHeat = 0.;
@@ -357,6 +361,7 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 	energyTable.push_back(Format::Number(-60. * firingEnergy));
 	heatTable.push_back(Format::Number(60. * firingHeat));
 
+	// Add energy and heat when doing shield and hull repair to the table.
 	attributesHeight += 20;
 	double shieldEnergy = (hasShieldRegen) ? attributes.Get("shield energy")
 		* (1. + attributes.Get("shield energy multiplier")) : 0.;
@@ -371,6 +376,26 @@ void ShipInfoDisplay::UpdateAttributes(const Ship &ship, const PlayerInfo &playe
 		* (1. + attributes.Get("hull heat multiplier")) : 0.;
 	heatTable.push_back(Format::Number(60. * (shieldHeat + hullHeat)));
 
+	if(scrollingPanel)
+	{
+		// Add up the maximum possible changes and add the total to the table.
+		attributesHeight += 20;
+		const double overallEnergy = idleEnergyPerFrame
+			- movingEnergyPerFrame
+			- firingEnergy
+			- shieldEnergy
+			- hullEnergy;
+		const double overallHeat = idleHeatPerFrame
+			+ movingHeatPerFrame
+			+ firingHeat
+			+ shieldHeat
+			+ hullHeat;
+		tableLabels.push_back("net change:");
+		energyTable.push_back(Format::Number(60. * overallEnergy));
+		heatTable.push_back(Format::Number(60. * overallHeat));
+	}
+
+	// Add maximum values of energy and heat to the table.
 	attributesHeight += 20;
 	const double maxEnergy = attributes.Get("energy capacity");
 	const double maxHeat = 60. * ship.HeatDissipation() * ship.MaximumHeat();
@@ -418,7 +443,7 @@ void ShipInfoDisplay::UpdateOutfits(const Ship &ship, const PlayerInfo &player,
 
 	int64_t totalCost = depreciation.Value(ship, player.GetDate().DaysSinceEpoch());
 	int64_t chassisCost = depreciation.Value(
-		GameData::Ships().Get(ship.ModelName()),
+		GameData::Ships().Get(ship.TrueModelName()),
 		player.GetDate().DaysSinceEpoch());
 	saleLabels.clear();
 	saleValues.clear();
