@@ -7,13 +7,15 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "SpriteQueue.h"
 
-#include "ImageBuffer.h"
-#include "Mask.h"
+#include "ImageSet.h"
 #include "Sprite.h"
 #include "SpriteSet.h"
 
@@ -23,18 +25,15 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
-	// Check if the given image is an @2x image.
-	bool Is2x(const string &path)
-	{
-		size_t len = path.length();
-		return (len > 7 && path[len - 7] == '@' && path[len - 6] == '2' && path[len - 5] == 'x');
-	}
+	// The maximum number of loaded images in the queue that are
+	// waiting to be uploaded to the GPU.
+	constexpr int MAX_QUEUE = 100;
 }
 
 
 
+// Constructor, which allocates worker threads.
 SpriteQueue::SpriteQueue()
-	: added(0), completed(0)
 {
 	threads.resize(max(4u, thread::hardware_concurrency()));
 	for(thread &t : threads)
@@ -43,6 +42,7 @@ SpriteQueue::SpriteQueue()
 
 
 
+// Destructor, which waits for all worker threads to wrap up.
 SpriteQueue::~SpriteQueue()
 {
 	{
@@ -57,18 +57,15 @@ SpriteQueue::~SpriteQueue()
 
 
 // Add a sprite to load.
-void SpriteQueue::Add(const string &name, const string &path)
+void SpriteQueue::Add(const shared_ptr<ImageSet> &images)
 {
-	Sprite *sprite = SpriteSet::Modify(name);
 	{
 		lock_guard<mutex> lock(readMutex);
 		// Do nothing if we are destroying the queue already.
 		if(added < 0)
 			return;
-		
-		bool is2x = Is2x(path);
-		int &frame = (is2x ? count2x[name] : count[name]);
-		toRead.emplace(sprite, name, path, frame++, is2x);
+
+		toRead.push(images);
 		++added;
 	}
 	readCondition.notify_one();
@@ -79,39 +76,47 @@ void SpriteQueue::Add(const string &name, const string &path)
 // Unload the texture for the given sprite (to free up memory).
 void SpriteQueue::Unload(const string &name)
 {
-	{
-		lock_guard<mutex> lock(readMutex);
-		count2x[name] = 0;
-		count[name] = 0;
-	}
-	
 	unique_lock<mutex> lock(loadMutex);
 	toUnload.push(name);
 }
 
 
 
-// Find out our percent completion.
-double SpriteQueue::Progress() const
+// Determine the fraction of sprites uploaded to the GPU.
+double SpriteQueue::GetProgress() const
+{
+	// Wait until we have completed loading of as many sprites as we have added.
+	// The value of "added" is protected by readMutex.
+	unique_lock<mutex> readLock(readMutex);
+	// Special cases: we're bailing out, or we are done.
+	if(added <= 0 || added == completed)
+		return 1.;
+	return static_cast<double>(completed) / static_cast<double>(added);
+}
+
+
+
+void SpriteQueue::UploadSprites()
 {
 	unique_lock<mutex> lock(loadMutex);
-	return DoLoad(lock);
+	DoLoad(lock);
 }
 
 
 
 // Finish loading.
-void SpriteQueue::Finish() const
+void SpriteQueue::Finish()
 {
 	// Loop until done loading.
 	while(true)
 	{
 		unique_lock<mutex> lock(loadMutex);
-		
+
 		// Load whatever is already queued up for loading.
-		if(DoLoad(lock) == 1.)
+		DoLoad(lock);
+		if(GetProgress() == 1.)
 			break;
-		
+
 		// We still have sprites to upload, but none of them have been read from
 		// disk yet. Wait until one arrives.
 		loadCondition.wait(lock);
@@ -134,83 +139,66 @@ void SpriteQueue::operator()()
 				return;
 			if(toRead.empty())
 				break;
-			
-			Item item = toRead.front();
+			{
+				// Stop loading images so that the main thread can keep up.
+				unique_lock<mutex> lock(loadMutex);
+				if(toLoad.size() > MAX_QUEUE)
+					break;
+			}
+
+			// Extract the one item we should work on reading right now.
+			shared_ptr<ImageSet> imageSet = toRead.front();
 			toRead.pop();
-			
+
+			// It's now safe to add to the lists.
 			lock.unlock();
-			
+
 			// Load the sprite.
-			item.image = ImageBuffer::Read(item.path);
-			// If sprite loading fails, just skip this sprite.
-			if(!item.image)
-			{
-				lock.lock();
-				continue;
-			}
-			// Don't ever create masks for @2x sprites; just use the ordinary
-			// sprite masks instead.
-			if(!item.is2x && (!item.name.compare(0, 5, "ship/") || !item.name.compare(0, 9, "asteroid/")))
-			{
-				item.mask = new Mask;
-				item.mask->Create(item.image);
-			}
-			
-			// Don't bother to copy the path, now that we've loaded the file.
-			item.name.clear();
-			item.path.clear();
+			// TODO: investigate catching exceptions from Load() (e.g. bad_alloc), to enable
+			// the UI thread to display a message prior to terminating the process.
+			imageSet->Load();
+
 			{
 				// The texture must be uploaded to OpenGL in the main thread.
 				unique_lock<mutex> lock(loadMutex);
-				toLoad.push(item);
+				toLoad.push(imageSet);
 			}
 			loadCondition.notify_one();
-			
+
 			lock.lock();
 		}
-		
+
 		readCondition.wait(lock);
 	}
 }
 
 
-double SpriteQueue::DoLoad(unique_lock<mutex> &lock) const
+
+void SpriteQueue::DoLoad(unique_lock<mutex> &lock)
 {
 	while(!toUnload.empty())
 	{
 		Sprite *sprite = SpriteSet::Modify(toUnload.front());
 		toUnload.pop();
-		
+
 		lock.unlock();
 		sprite->Unload();
 		lock.lock();
 	}
-	
-	for(int i = 0; !toLoad.empty() && i < 100; ++i)
+
+	for(int i = 0; !toLoad.empty() && i < MAX_QUEUE; ++i)
 	{
-		Item item = toLoad.front();
+		// Extract the one item we should work on uploading right now.
+		shared_ptr<ImageSet> imageSet = toLoad.front();
 		toLoad.pop();
-		
+
+		// It's now safe to modify the lists.
 		lock.unlock();
-		
-		item.sprite->AddFrame(item.frame, item.image, item.mask, item.is2x);
-		
+
+		readCondition.notify_one();
+		imageSet->Upload(SpriteSet::Modify(imageSet->Name()));
+
 		lock.lock();
 		++completed;
 	}
-	
-	// Wait until we have completed loading of as many sprites as we have added.
-	// The value of "added" is protected by readMutex.
-	unique_lock<mutex> readLock(readMutex);
-	// Special cases: we're bailing out, or we are done.
-	if(added <= 0 || added == completed)
-		return 1.;
-	return static_cast<double>(completed) / static_cast<double>(added);
-}
-
-
-
-SpriteQueue::Item::Item(Sprite *sprite, const string &name, const string &path, int frame, bool is2x)
-	: sprite(sprite), name(name), path(path), image(nullptr), mask(nullptr), frame(frame), is2x(is2x)
-{
 }
