@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef SPRITE_SET_H_
@@ -27,11 +30,11 @@ class SpriteSet {
 public:
 	// Get a pointer to the sprite data with the given name.
 	static const Sprite *Get(const std::string &name);
-	
-	// Inspect the sprite map and return any paths that loaded no data.
-	static std::set<std::string> CheckReferences();
-	
-	
+
+	// Inspect the sprite map and warn if some images contain no data.
+	static void CheckReferences();
+
+
 private:
 	// Only SpriteQueue is allowed to modify the sprites.
 	friend class SpriteQueue;
