@@ -16,7 +16,6 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 #include "StartConditions.h"
 
 #include "DataNode.h"
-#include "DataWriter.h"
 #include "text/Format.h"
 #include "GameData.h"
 #include "Logger.h"
@@ -88,7 +87,7 @@ void StartConditions::Load(const DataNode &node)
 				ships.clear();
 			else if(key == "ship" && hasValue)
 				ships.erase(remove_if(ships.begin(), ships.end(),
-					[&value](const Ship &s) noexcept -> bool { return s.ModelName() == value; }),
+					[&value](const Ship &s) noexcept -> bool { return s.TrueModelName() == value; }),
 					ships.end());
 			else if(key == "conversation")
 				conversation = ExclusiveItem<Conversation>();
