@@ -175,6 +175,8 @@ void System::Load(const DataNode &node, Set<Planet> &planets)
 				hidden = false;
 			else if(key == "inaccessible")
 				inaccessible = false;
+			else if(key == "no raids")
+				noRaids = false;
 
 			// If not in "overwrite" mode, move on to the next node.
 			if(overwriteAll)
@@ -188,6 +190,8 @@ void System::Load(const DataNode &node, Set<Planet> &planets)
 			hidden = true;
 		else if(key == "inaccessible")
 			inaccessible = true;
+		else if(key == "no raids")
+			noRaids = true;
 		else if(key == "ramscoop")
 		{
 			for(const DataNode &grand : child)
@@ -270,6 +274,8 @@ void System::Load(const DataNode &node, Set<Planet> &planets)
 			else
 				fleets.emplace_back(fleet, child.Value(valueIndex + 1));
 		}
+		else if(key == "raid")
+			RaidFleet::Load(raidFleets, child, remove, valueIndex);
 		else if(key == "hazard")
 		{
 			const Hazard *hazard = GameData::Hazards().Get(value);
@@ -966,6 +972,15 @@ int System::MinimumFleetPeriod() const
 
 
 
+const vector<RaidFleet> &System::RaidFleets() const
+{
+	static const vector<RaidFleet> EMPTY;
+	// If the system defines its own raid fleets then those are used in lieu of the government's fleets.
+	return noRaids ? EMPTY : ((raidFleets.empty() && government) ? government->RaidFleets() : raidFleets);
+}
+
+
+
 void System::LoadObject(const DataNode &node, Set<Planet> &planets, int parent)
 {
 	int index = objects.size();
