@@ -18,6 +18,7 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 #include "Hazard.h"
 #include "Point.h"
+#include "RaidFleet.h"
 #include "RandomEvent.h"
 #include "Set.h"
 #include "StellarObject.h"
@@ -173,6 +174,8 @@ public:
 	// The smallest arrival period of a fleet (or 0 if no fleets arrive)
 	int MinimumFleetPeriod() const;
 
+	const std::vector<RaidFleet> &RaidFleets() const;
+
 
 private:
 	void LoadObject(const DataNode &node, Set<Planet> &planets, int parent = -1);
@@ -243,6 +246,9 @@ private:
 	double starfieldDensity = 1.;
 	int minimumFleetPeriod = 0;
 
+	std::vector<RaidFleet> raidFleets;
+	bool noRaids = false;
+
 	// The amount of additional distance that ships will arrive away from the
 	// system center when entering this system through a hyperspace link.
 	// Negative values are allowed, causing ships to jump beyond their target.
