@@ -19,9 +19,9 @@ this program. If not, see <https://www.gnu.org/licenses/>.
 
 
 // Constants to replace M_PI (which is not available on all operating systems).
-const double PI = 3.14159265358979323846;
-const double TO_RAD = PI / 180.;
-const double TO_DEG = 180. / PI;
+constexpr double PI = 3.14159265358979323846;
+constexpr double TO_RAD = PI / 180.;
+constexpr double TO_DEG = 180. / PI;
 
 
 
