@@ -177,7 +177,7 @@ def check_code_style(file, lines):
 # Appends the lists in the second tuple to the lists in the first tuple. Parameters:
 # first: the tuple where the lists are expanded
 # second: the tuple where the lists are not expanded
-# Returns the second tuple for re-use.
+# Returns the second tuple for reuse.
 def join(first, second):
 	for (list1, list2) in zip(first, second):
 		list1 += list2
@@ -560,7 +560,7 @@ def check_include(sanitized_lines, original_lines, file):
 				group_lines[i] = line
 		for i in range(len(group) - 1):
 			if group_lines[i].lower() > group_lines[i + 1].lower():
-				warnings.append(Warning(group_lines[i], group[i] + 1, "includes are not in alphabetical order"))
+				errors.append(Error(group_lines[i], group[i] + 1, "includes are not in alphabetical order"))
 	return errors, warnings
 
 
